#!/usr/bin/env python3
"""
會話清理管理器
==============

統一管理 Web 會話的清理策略、統計和性能監控。
與內存監控系統深度集成，提供智能清理決策。
"""

import threading
import time
from collections.abc import Callable
from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import Any

from ...debug import web_debug_log as debug_log
from ...utils.error_handler import ErrorHandler, ErrorType
from ..models.feedback_session import CleanupReason, SessionStatus


@dataclass
class CleanupPolicy:
    """清理策略配置"""

    max_idle_time: int = 1800  # 最大空閒時間（秒）
    max_session_age: int = 7200  # 最大會話年齡（秒）
    max_sessions: int = 10  # 最大會話數量
    cleanup_interval: int = 300  # 清理間隔（秒）
    memory_pressure_threshold: float = 0.8  # 內存壓力閾值
    enable_auto_cleanup: bool = True  # 啟用自動清理
    preserve_active_session: bool = True  # 保護活躍會話


@dataclass
class CleanupStats:
    """清理統計數據"""

    total_cleanups: int = 0
    expired_cleanups: int = 0
    memory_pressure_cleanups: int = 0
    manual_cleanups: int = 0
    auto_cleanups: int = 0
    total_sessions_cleaned: int = 0
    total_cleanup_time: float = 0.0
    average_cleanup_time: float = 0.0
    last_cleanup_time: datetime | None = None
    cleanup_efficiency: float = 0.0  # 清理效率（清理的會話數/總會話數）


class CleanupTrigger(Enum):
    """清理觸發器類型"""

    AUTO = "auto"  # 自動清理
    MEMORY_PRESSURE = "memory_pressure"  # 內存壓力
    MANUAL = "manual"  # 手動清理
    EXPIRED = "expired"  # 過期清理
    CAPACITY = "capacity"  # 容量限制


class SessionCleanupManager:
    """會話清理管理器"""

    def __init__(self, web_ui_manager, policy: CleanupPolicy | None = None):
        """
        初始化會話清理管理器

        Args:
            web_ui_manager: WebUIManager 實例
            policy: 清理策略配置
        """
        self.web_ui_manager = web_ui_manager
        self.policy = policy or CleanupPolicy()
        self.stats = CleanupStats()

        # 清理狀態
        self.is_running = False
        self.cleanup_thread: threading.Thread | None = None
        self._stop_event = threading.Event()

        # 回調函數
        self.cleanup_callbacks: list[Callable] = []
        self.stats_callbacks: list[Callable] = []

        # 清理歷史記錄
        self.cleanup_history: list[dict[str, Any]] = []
        self.max_history = 100

        debug_log("SessionCleanupManager 初始化完成")

    def start_auto_cleanup(self) -> bool:
        """啟動自動清理"""
        if not self.policy.enable_auto_cleanup:
            debug_log("自動清理已禁用")
            return False

        if self.is_running:
            debug_log("自動清理已在運行")
            return True

        try:
            self.is_running = True
            self._stop_event.clear()

            self.cleanup_thread = threading.Thread(
                target=self._auto_cleanup_loop,
                name="SessionCleanupManager",
                daemon=True,
            )
            self.cleanup_thread.start()

            debug_log(f"自動清理已啟動，間隔 {self.policy.cleanup_interval} 秒")
            return True

        except Exception as e:
            self.is_running = False
            error_id = ErrorHandler.log_error_with_context(
                e, context={"operation": "啟動自動清理"}, error_type=ErrorType.SYSTEM
            )
            debug_log(f"啟動自動清理失敗 [錯誤ID: {error_id}]: {e}")
            return False

    def stop_auto_cleanup(self) -> bool:
        """停止自動清理"""
        if not self.is_running:
            debug_log("自動清理未在運行")
            return True

        try:
            self.is_running = False
            self._stop_event.set()

            if self.cleanup_thread and self.cleanup_thread.is_alive():
                self.cleanup_thread.join(timeout=5)

            debug_log("自動清理已停止")
            return True

        except Exception as e:
            error_id = ErrorHandler.log_error_with_context(
                e, context={"operation": "停止自動清理"}, error_type=ErrorType.SYSTEM
            )
            debug_log(f"停止自動清理失敗 [錯誤ID: {error_id}]: {e}")
            return False

    def _auto_cleanup_loop(self):
        """自動清理主循環"""
        debug_log("自動清理循環開始")

        while not self._stop_event.is_set():
            try:
                # 執行清理檢查
                self._perform_auto_cleanup()

                # 等待下次清理
                if self._stop_event.wait(self.policy.cleanup_interval):
                    break

            except Exception as e:
                error_id = ErrorHandler.log_error_with_context(
                    e,
                    context={"operation": "自動清理循環"},
                    error_type=ErrorType.SYSTEM,
                )
                debug_log(f"自動清理循環錯誤 [錯誤ID: {error_id}]: {e}")

                # 發生錯誤時等待較短時間後重試
                if self._stop_event.wait(30):
                    break

        debug_log("自動清理循環結束")

    def _perform_auto_cleanup(self):
        """執行自動清理"""
        cleanup_start_time = time.time()
        cleaned_sessions = 0

        try:
            # 1. 檢查會話數量限制
            if len(self.web_ui_manager.sessions) > self.policy.max_sessions:
                cleaned = self._cleanup_by_capacity()
                cleaned_sessions += cleaned
                debug_log(f"容量限制清理了 {cleaned} 個會話")

            # 2. 清理過期會話
            cleaned = self._cleanup_expired_sessions()
            cleaned_sessions += cleaned

            # 3. 清理空閒會話
            cleaned = self._cleanup_idle_sessions()
            cleaned_sessions += cleaned

            # 4. 更新統計
            cleanup_duration = time.time() - cleanup_start_time
            self._update_cleanup_stats(
                CleanupTrigger.AUTO, cleaned_sessions, cleanup_duration
            )

            if cleaned_sessions > 0:
                debug_log(
                    f"自動清理完成，清理了 {cleaned_sessions} 個會話，耗時: {cleanup_duration:.2f}秒"
                )

        except Exception as e:
            error_id = ErrorHandler.log_error_with_context(
                e, context={"operation": "執行自動清理"}, error_type=ErrorType.SYSTEM
            )
            debug_log(f"執行自動清理失敗 [錯誤ID: {error_id}]: {e}")

    def trigger_cleanup(self, trigger: CleanupTrigger, force: bool = False) -> int:
        """觸發清理操作"""
        cleanup_start_time = time.time()
        cleaned_sessions = 0

        try:
            debug_log(f"觸發清理操作，觸發器: {trigger.value}，強制: {force}")

            if trigger == CleanupTrigger.MEMORY_PRESSURE:
                cleaned_sessions = (
                    self.web_ui_manager.cleanup_sessions_by_memory_pressure(force)
                )
            elif trigger == CleanupTrigger.EXPIRED:
                cleaned_sessions = self.web_ui_manager.cleanup_expired_sessions()
            elif trigger == CleanupTrigger.CAPACITY:
                cleaned_sessions = self._cleanup_by_capacity()
            elif trigger == CleanupTrigger.MANUAL:
                # 手動清理：組合多種策略
                cleaned_sessions += self.web_ui_manager.cleanup_expired_sessions()
                if force:
                    cleaned_sessions += (
                        self.web_ui_manager.cleanup_sessions_by_memory_pressure(force)
                    )
            else:
                # 自動清理
                self._perform_auto_cleanup()
                return 0  # 統計已在 _perform_auto_cleanup 中更新

            # 更新統計
            cleanup_duration = time.time() - cleanup_start_time
            self._update_cleanup_stats(trigger, cleaned_sessions, cleanup_duration)

            debug_log(
                f"清理操作完成，清理了 {cleaned_sessions} 個會話，耗時: {cleanup_duration:.2f}秒"
            )
            return cleaned_sessions

        except Exception as e:
            error_id = ErrorHandler.log_error_with_context(
                e,
                context={
                    "operation": "觸發清理",
                    "trigger": trigger.value,
                    "force": force,
                },
                error_type=ErrorType.SYSTEM,
            )
            debug_log(f"觸發清理操作失敗 [錯誤ID: {error_id}]: {e}")
            return 0

    def _cleanup_by_capacity(self) -> int:
        """根據容量限制清理會話"""
        sessions = self.web_ui_manager.sessions
        if len(sessions) <= self.policy.max_sessions:
            return 0

        # 計算需要清理的會話數量
        excess_count = len(sessions) - self.policy.max_sessions

        # 按優先級排序會話（優先清理舊的、非活躍的會話）
        session_priorities = []
        for session_id, session in sessions.items():
            # 跳過當前活躍會話（如果啟用保護）
            if (
                self.policy.preserve_active_session
                and self.web_ui_manager.current_session
                and session.session_id == self.web_ui_manager.current_session.session_id
            ):
                continue

            # 計算優先級分數（分數越高越優先清理）
            priority_score = 0

            # 狀態優先級
            if session.status in [
                SessionStatus.COMPLETED,
                SessionStatus.ERROR,
                SessionStatus.TIMEOUT,
            ]:
                priority_score += 100
            elif session.status == SessionStatus.FEEDBACK_SUBMITTED:
                priority_score += 50

            # 年齡優先級
            age = session.get_age()
            priority_score += age / 60  # 每分鐘加1分

            # 空閒時間優先級
            idle_time = session.get_idle_time()
            priority_score += idle_time / 30  # 每30秒加1分

            session_priorities.append((session_id, session, priority_score))

        # 按優先級排序並清理
        session_priorities.sort(key=lambda x: x[2], reverse=True)
        cleaned_count = 0

        for i in range(min(excess_count, len(session_priorities))):
            session_id, session, _ = session_priorities[i]
            try:
                session._cleanup_sync_enhanced(CleanupReason.MANUAL)
                del self.web_ui_manager.sessions[session_id]
                cleaned_count += 1
            except Exception as e:
                debug_log(f"容量清理會話 {session_id} 失敗: {e}")

        return cleaned_count

    def _cleanup_expired_sessions(self) -> int:
        """清理過期會話"""
        expired_sessions = []

        for session_id, session in self.web_ui_manager.sessions.items():
            # 檢查是否過期
            if session.is_expired() or session.get_age() > self.policy.max_session_age:
                expired_sessions.append(session_id)

        # 清理過期會話
        cleaned_count = 0
        for session_id in expired_sessions:
            try:
                session = self.web_ui_manager.sessions.get(session_id)
                if session:
                    session._cleanup_sync_enhanced(CleanupReason.EXPIRED)
                    del self.web_ui_manager.sessions[session_id]
                    cleaned_count += 1

                    # 如果清理的是當前活躍會話，清空當前會話
                    if (
                        self.web_ui_manager.current_session
                        and self.web_ui_manager.current_session.session_id == session_id
                    ):
                        self.web_ui_manager.current_session = None

            except Exception as e:
                debug_log(f"清理過期會話 {session_id} 失敗: {e}")

        return cleaned_count

    def _cleanup_idle_sessions(self) -> int:
        """清理空閒會話"""
        idle_sessions = []

        for session_id, session in self.web_ui_manager.sessions.items():
            # 跳過當前活躍會話（如果啟用保護）
            if (
                self.policy.preserve_active_session
                and self.web_ui_manager.current_session
                and session.session_id == self.web_ui_manager.current_session.session_id
            ):
                continue

            # 檢查是否空閒時間過長
            if session.get_idle_time() > self.policy.max_idle_time:
                idle_sessions.append(session_id)

        # 清理空閒會話
        cleaned_count = 0
        for session_id in idle_sessions:
            try:
                session = self.web_ui_manager.sessions.get(session_id)
                if session:
                    session._cleanup_sync_enhanced(CleanupReason.EXPIRED)
                    del self.web_ui_manager.sessions[session_id]
                    cleaned_count += 1

            except Exception as e:
                debug_log(f"清理空閒會話 {session_id} 失敗: {e}")

        return cleaned_count

    def _update_cleanup_stats(
        self, trigger: CleanupTrigger, cleaned_count: int, duration: float
    ):
        """更新清理統計"""
        self.stats.total_cleanups += 1
        self.stats.total_sessions_cleaned += cleaned_count
        self.stats.total_cleanup_time += duration
        self.stats.last_cleanup_time = datetime.now()

        # 更新平均清理時間
        if self.stats.total_cleanups > 0:
            self.stats.average_cleanup_time = (
                self.stats.total_cleanup_time / self.stats.total_cleanups
            )

        # 更新清理效率
        total_sessions = len(self.web_ui_manager.sessions) + cleaned_count
        if total_sessions > 0:
            self.stats.cleanup_efficiency = cleaned_count / total_sessions

        # 根據觸發器類型更新統計
        if trigger == CleanupTrigger.AUTO:
            self.stats.auto_cleanups += 1
        elif trigger == CleanupTrigger.MEMORY_PRESSURE:
            self.stats.memory_pressure_cleanups += 1
        elif trigger == CleanupTrigger.EXPIRED:
            self.stats.expired_cleanups += 1
        elif trigger == CleanupTrigger.MANUAL:
            self.stats.manual_cleanups += 1

        # 記錄清理歷史
        cleanup_record = {
            "timestamp": datetime.now().isoformat(),
            "trigger": trigger.value,
            "cleaned_count": cleaned_count,
            "duration": duration,
            "total_sessions_before": total_sessions,
            "total_sessions_after": len(self.web_ui_manager.sessions),
        }

        self.cleanup_history.append(cleanup_record)

        # 限制歷史記錄數量
        if len(self.cleanup_history) > self.max_history:
            self.cleanup_history = self.cleanup_history[-self.max_history :]

        # 調用統計回調
        for callback in self.stats_callbacks:
            try:
                callback(self.stats, cleanup_record)
            except Exception as e:
                debug_log(f"統計回調執行失敗: {e}")

    def get_cleanup_statistics(self) -> dict[str, Any]:
        """獲取清理統計數據"""
        stats_dict = {
            "total_cleanups": self.stats.total_cleanups,
            "expired_cleanups": self.stats.expired_cleanups,
            "memory_pressure_cleanups": self.stats.memory_pressure_cleanups,
            "manual_cleanups": self.stats.manual_cleanups,
            "auto_cleanups": self.stats.auto_cleanups,
            "total_sessions_cleaned": self.stats.total_sessions_cleaned,
            "total_cleanup_time": round(self.stats.total_cleanup_time, 2),
            "average_cleanup_time": round(self.stats.average_cleanup_time, 2),
            "cleanup_efficiency": round(self.stats.cleanup_efficiency, 3),
            "last_cleanup_time": self.stats.last_cleanup_time.isoformat()
            if self.stats.last_cleanup_time
            else None,
            "is_auto_cleanup_running": self.is_running,
            "current_sessions": len(self.web_ui_manager.sessions),
            "policy": {
                "max_idle_time": self.policy.max_idle_time,
                "max_session_age": self.policy.max_session_age,
                "max_sessions": self.policy.max_sessions,
                "cleanup_interval": self.policy.cleanup_interval,
                "enable_auto_cleanup": self.policy.enable_auto_cleanup,
                "preserve_active_session": self.policy.preserve_active_session,
            },
        }

        return stats_dict

    def get_cleanup_history(self, limit: int = 20) -> list[dict[str, Any]]:
        """獲取清理歷史記錄"""
        return self.cleanup_history[-limit:] if self.cleanup_history else []

    def add_cleanup_callback(self, callback: Callable):
        """添加清理回調函數"""
        if callback not in self.cleanup_callbacks:
            self.cleanup_callbacks.append(callback)
            debug_log("添加清理回調函數")

    def add_stats_callback(self, callback: Callable):
        """添加統計回調函數"""
        if callback not in self.stats_callbacks:
            self.stats_callbacks.append(callback)
            debug_log("添加統計回調函數")

    def update_policy(self, **kwargs):
        """更新清理策略"""
        for key, value in kwargs.items():
            if hasattr(self.policy, key):
                setattr(self.policy, key, value)
                debug_log(f"更新清理策略 {key} = {value}")
            else:
                debug_log(f"未知的策略參數: {key}")

    def reset_stats(self):
        """重置統計數據"""
        self.stats = CleanupStats()
        self.cleanup_history.clear()
        debug_log("清理統計數據已重置")

    def force_cleanup_all(self, exclude_current: bool = True) -> int:
        """強制清理所有會話"""
        sessions_to_clean = []

        for session_id, session in self.web_ui_manager.sessions.items():
            # 是否排除當前活躍會話
            if (
                exclude_current
                and self.web_ui_manager.current_session
                and session.session_id == self.web_ui_manager.current_session.session_id
            ):
                continue
            sessions_to_clean.append(session_id)

        # 清理會話
        cleaned_count = 0
        for session_id in sessions_to_clean:
            try:
                session = self.web_ui_manager.sessions.get(session_id)
                if session:
                    session._cleanup_sync_enhanced(CleanupReason.MANUAL)
                    del self.web_ui_manager.sessions[session_id]
                    cleaned_count += 1
            except Exception as e:
                debug_log(f"強制清理會話 {session_id} 失敗: {e}")

        # 更新統計
        self._update_cleanup_stats(CleanupTrigger.MANUAL, cleaned_count, 0.0)

        debug_log(f"強制清理完成，清理了 {cleaned_count} 個會話")
        return cleaned_count



#!/usr/bin/env python3
"""
壓縮配置管理器
==============

管理 Web UI 的 Gzip 壓縮配置和靜態文件緩存策略。
支援可配置的壓縮參數和性能優化選項。
"""

import os
from dataclasses import dataclass, field
from typing import Any


@dataclass
class CompressionConfig:
    """壓縮配置類"""

    # Gzip 壓縮設定
    minimum_size: int = 1000  # 最小壓縮大小（bytes）
    compression_level: int = 6  # 壓縮級別 (1-9, 6為平衡點)

    # 緩存設定
    static_cache_max_age: int = 3600  # 靜態文件緩存時間（秒）
    api_cache_max_age: int = 0  # API 響應緩存時間（秒，0表示不緩存）

    # 支援的 MIME 類型
    compressible_types: list[str] = field(default_factory=list)

    # 排除的路徑
    exclude_paths: list[str] = field(default_factory=list)

    def __post_init__(self):
        """初始化後處理"""
        if not self.compressible_types:
            self.compressible_types = [
                "text/html",
                "text/css",
                "text/javascript",
                "text/plain",
                "application/json",
                "application/javascript",
                "application/xml",
                "application/rss+xml",
                "application/atom+xml",
                "image/svg+xml",
            ]

        if not self.exclude_paths:
            self.exclude_paths = [
                "/ws",  # WebSocket 連接
                "/api/ws",  # WebSocket API
                "/health",  # 健康檢查
            ]

    @classmethod
    def from_env(cls) -> "CompressionConfig":
        """從環境變數創建配置"""
        return cls(
            minimum_size=int(os.getenv("MCP_GZIP_MIN_SIZE", "1000")),
            compression_level=int(os.getenv("MCP_GZIP_LEVEL", "6")),
            static_cache_max_age=int(os.getenv("MCP_STATIC_CACHE_AGE", "3600")),
            api_cache_max_age=int(os.getenv("MCP_API_CACHE_AGE", "0")),
        )

    def should_compress(self, content_type: str, content_length: int) -> bool:
        """判斷是否應該壓縮"""
        if content_length < self.minimum_size:
            return False

        if not content_type:
            return False

        # 檢查 MIME 類型
        for mime_type in self.compressible_types:
            if content_type.startswith(mime_type):
                return True

        return False

    def should_exclude_path(self, path: str) -> bool:
        """判斷路徑是否應該排除壓縮"""
        for exclude_path in self.exclude_paths:
            if path.startswith(exclude_path):
                return True
        return False

    def get_cache_headers(self, path: str) -> dict[str, str]:
        """獲取緩存頭"""
        headers = {}

        if path.startswith("/static/"):
            # 靜態文件緩存
            headers["Cache-Control"] = f"public, max-age={self.static_cache_max_age}"
            headers["Expires"] = self._get_expires_header(self.static_cache_max_age)
        elif path.startswith("/api/") and self.api_cache_max_age > 0:
            # API 緩存（如果啟用）
            headers["Cache-Control"] = f"public, max-age={self.api_cache_max_age}"
            headers["Expires"] = self._get_expires_header(self.api_cache_max_age)
        else:
            # 其他路徑不緩存
            headers["Cache-Control"] = "no-cache, no-store, must-revalidate"
            headers["Pragma"] = "no-cache"
            headers["Expires"] = "0"

        return headers

    def _get_expires_header(self, max_age: int) -> str:
        """生成 Expires 頭"""
        from datetime import datetime, timedelta

        expires_time = datetime.utcnow() + timedelta(seconds=max_age)
        return expires_time.strftime("%a, %d %b %Y %H:%M:%S GMT")

    def get_compression_stats(self) -> dict[str, Any]:
        """獲取壓縮配置統計"""
        return {
            "minimum_size": self.minimum_size,
            "compression_level": self.compression_level,
            "static_cache_max_age": self.static_cache_max_age,
            "compressible_types_count": len(self.compressible_types),
            "exclude_paths_count": len(self.exclude_paths),
            "compressible_types": self.compressible_types,
            "exclude_paths": self.exclude_paths,
        }


class CompressionManager:
    """壓縮管理器"""

    def __init__(self, config: CompressionConfig | None = None):
        self.config = config or CompressionConfig.from_env()
        self._stats = {
            "requests_total": 0,
            "requests_compressed": 0,
            "bytes_original": 0,
            "bytes_compressed": 0,
            "compression_ratio": 0.0,
        }

    def update_stats(
        self, original_size: int, compressed_size: int, was_compressed: bool
    ):
        """更新壓縮統計"""
        self._stats["requests_total"] += 1
        self._stats["bytes_original"] += original_size

        if was_compressed:
            self._stats["requests_compressed"] += 1
            self._stats["bytes_compressed"] += compressed_size
        else:
            self._stats["bytes_compressed"] += original_size

        # 計算壓縮比率
        if self._stats["bytes_original"] > 0:
            self._stats["compression_ratio"] = (
                1 - self._stats["bytes_compressed"] / self._stats["bytes_original"]
            ) * 100

    def get_stats(self) -> dict[str, Any]:
        """獲取壓縮統計"""
        stats = self._stats.copy()
        stats["compression_percentage"] = (
            self._stats["requests_compressed"]
            / max(self._stats["requests_total"], 1)
            * 100
        )
        return stats

    def reset_stats(self):
        """重置統計"""
        self._stats = {
            "requests_total": 0,
            "requests_compressed": 0,
            "bytes_original": 0,
            "bytes_compressed": 0,
            "compression_ratio": 0.0,
        }


# 全域壓縮管理器實例
_compression_manager: CompressionManager | None = None


def get_compression_manager() -> CompressionManager:
    """獲取全域壓縮管理器實例"""
    global _compression_manager
    if _compression_manager is None:
        _compression_manager = CompressionManager()
    return _compression_manager



#!/usr/bin/env python3
"""
MCP Interactive Feedback Enhanced - 主程式入口
==============================================

此檔案允許套件透過 `python -m mcp_feedback_enhanced` 執行。

使用方法:
  python -m mcp_feedback_enhanced        # 啟動 MCP 伺服器
  python -m mcp_feedback_enhanced test   # 執行測試
"""

import argparse
import asyncio
import os
import sys
import warnings


# 抑制 Windows 上的 asyncio ResourceWarning
if sys.platform == "win32":
    warnings.filterwarnings(
        "ignore", category=ResourceWarning, message=".*unclosed transport.*"
    )
    warnings.filterwarnings("ignore", category=ResourceWarning, message=".*unclosed.*")

    # 設置 asyncio 事件循環策略以減少警告
    try:
        asyncio.set_event_loop_policy(asyncio.WindowsProactorEventLoopPolicy())
    except AttributeError:
        pass


def main():
    """主程式入口點"""
    parser = argparse.ArgumentParser(
        description="MCP Feedback Enhanced Enhanced - 互動式回饋收集 MCP 伺服器"
    )

    subparsers = parser.add_subparsers(dest="command", help="可用命令")

    # 伺服器命令（預設）
    subparsers.add_parser("server", help="啟動 MCP 伺服器（預設）")

    # 測試命令
    test_parser = subparsers.add_parser("test", help="執行測試")
    test_parser.add_argument(
        "--web", action="store_true", help="測試 Web UI (自動持續運行)"
    )
    test_parser.add_argument(
        "--desktop", action="store_true", help="啟動桌面應用程式模式"
    )
    test_parser.add_argument(
        "--timeout", type=int, default=60, help="測試超時時間 (秒)"
    )

    # 版本命令
    subparsers.add_parser("version", help="顯示版本資訊")

    args = parser.parse_args()

    if args.command == "test":
        run_tests(args)
    elif args.command == "version":
        show_version()
    elif args.command == "server" or args.command is None:
        run_server()
    else:
        # 不應該到達這裡
        parser.print_help()
        sys.exit(1)


def run_server():
    """啟動 MCP 伺服器"""
    from .server import main as server_main

    return server_main()


def run_tests(args):
    """執行測試"""
    # 啟用調試模式以顯示測試過程
    os.environ["MCP_DEBUG"] = "true"

    # 在 Windows 上抑制 asyncio 警告
    if sys.platform == "win32":
        import warnings

        # 設置更全面的警告抑制
        os.environ["PYTHONWARNINGS"] = (
            "ignore::ResourceWarning,ignore::DeprecationWarning"
        )
        warnings.filterwarnings("ignore", category=ResourceWarning)
        warnings.filterwarnings("ignore", message=".*unclosed transport.*")
        warnings.filterwarnings("ignore", message=".*I/O operation on closed pipe.*")
        warnings.filterwarnings("ignore", message=".*unclosed.*")
        # 抑制 asyncio 相關的所有警告
        warnings.filterwarnings("ignore", module="asyncio.*")

    if args.web:
        print("🧪 執行 Web UI 測試...")
        success = test_web_ui_simple()
        if not success:
            sys.exit(1)
    elif args.desktop:
        print("🖥️ 啟動桌面應用程式...")
        success = test_desktop_app()
        if not success:
            sys.exit(1)
    else:
        print("❌ 測試功能已簡化")
        print("💡 可用的測試選項：")
        print("  --web         測試 Web UI")
        print("  --desktop     啟動桌面應用程式")
        print("💡 對於開發者：使用 'uv run pytest' 執行完整測試")
        sys.exit(1)


def test_web_ui_simple():
    """簡單的 Web UI 測試"""
    try:
        import tempfile
        import time
        import webbrowser

        from .web.main import WebUIManager

        # 設置測試模式，禁用自動清理避免權限問題
        os.environ["MCP_TEST_MODE"] = "true"
        # 設置更高的端口範圍避免系統保留端口
        os.environ["MCP_WEB_PORT"] = "9765"

        print("🔧 創建 Web UI 管理器...")
        manager = WebUIManager(host="127.0.0.1")  # 使用動態端口分配

        print("🔧 創建測試會話...")
        with tempfile.TemporaryDirectory() as temp_dir:
            markdown_test_content = """# Web UI 測試 - Markdown 渲染功能

## 🎯 測試目標
驗證 **combinedSummaryContent** 區域的 Markdown 語法顯示功能

### ✨ 支援的語法特性

#### 文字格式
- **粗體文字** 使用雙星號
- *斜體文字* 使用單星號
- ~~刪除線文字~~ 使用雙波浪號
- `行內程式碼` 使用反引號

#### 程式碼區塊
```javascript
// JavaScript 範例
function renderMarkdown(content) {
    return marked.parse(content);
}
```

```python
# Python 範例
def process_feedback(data):
    return {"status": "success", "data": data}
```

#### 列表功能
**無序列表：**
- 第一個項目
- 第二個項目
  - 巢狀項目 1
  - 巢狀項目 2
- 第三個項目

**有序列表：**
1. 初始化 Markdown 渲染器
2. 載入 marked.js 和 DOMPurify
3. 配置安全選項
4. 渲染內容

#### 連結和引用
- 專案連結：[MCP Feedback Enhanced](https://github.com/example/mcp-feedback-enhanced)
- 文檔連結：[Marked.js 官方文檔](https://marked.js.org/)

> **重要提示：** 所有 HTML 輸出都經過 DOMPurify 清理，確保安全性。

#### 表格範例
| 功能 | 狀態 | 說明 |
|------|------|------|
| 標題渲染 | ✅ | 支援 H1-H6 |
| 程式碼高亮 | ✅ | 基本語法高亮 |
| 列表功能 | ✅ | 有序/無序列表 |
| 連結處理 | ✅ | 安全連結渲染 |

---

### 🔒 安全特性
- XSS 防護：使用 DOMPurify 清理
- 白名單標籤：僅允許安全的 HTML 標籤
- URL 驗證：限制允許的 URL 協議

### 📝 測試結果
如果您能看到上述內容以正確的格式顯示，表示 Markdown 渲染功能運作正常！"""

            created_session_id = manager.create_session(temp_dir, markdown_test_content)

            if created_session_id:
                print("✅ 會話創建成功")

                print("🚀 啟動 Web 服務器...")
                manager.start_server()
                time.sleep(5)  # 等待服務器完全啟動

                if (
                    manager.server_thread is not None
                    and manager.server_thread.is_alive()
                ):
                    print("✅ Web 服務器啟動成功")
                    url = f"http://{manager.host}:{manager.port}"
                    print(f"🌐 服務器運行在: {url}")

                    # 嘗試開啟瀏覽器
                    print("🌐 正在開啟瀏覽器...")
                    try:
                        webbrowser.open(url)
                        print("✅ 瀏覽器已開啟")
                    except Exception as e:
                        print(f"⚠️  無法自動開啟瀏覽器: {e}")
                        print(f"💡 請手動開啟瀏覽器並訪問: {url}")

                    print("📝 Web UI 測試完成，進入持續模式...")
                    print("💡 提示：服務器將持續運行，可在瀏覽器中測試互動功能")
                    print("💡 按 Ctrl+C 停止服務器")

                    try:
                        # 保持服務器運行
                        while True:
                            time.sleep(1)
                    except KeyboardInterrupt:
                        print("\n🛑 停止服務器...")
                        return True
                else:
                    print("❌ Web 服務器啟動失敗")
                    return False
            else:
                print("❌ 會話創建失敗")
                return False

    except Exception as e:
        print(f"❌ Web UI 測試失敗: {e}")
        import traceback

        traceback.print_exc()
        return False
    finally:
        # 清理測試環境變數
        os.environ.pop("MCP_TEST_MODE", None)
        os.environ.pop("MCP_WEB_PORT", None)


def test_desktop_app():
    """測試桌面應用程式"""
    try:
        print("🔧 檢查桌面應用程式依賴...")

        # 檢查是否有 Tauri 桌面模組
        try:
            import os
            import sys

            # 嘗試導入桌面應用程式模組
            def import_desktop_app():
                # 首先嘗試從發佈包位置導入
                try:
                    from .desktop_app import launch_desktop_app as desktop_func

                    print("✅ 找到發佈包中的桌面應用程式模組")
                    return desktop_func
                except ImportError:
                    print("🔍 發佈包中未找到桌面應用程式模組，嘗試開發環境...")

                # 回退到開發環境路徑
                tauri_python_path = os.path.join(
                    os.path.dirname(__file__), "..", "..", "src-tauri", "python"
                )
                if os.path.exists(tauri_python_path):
                    sys.path.insert(0, tauri_python_path)
                    print(f"✅ 找到 Tauri Python 模組路徑: {tauri_python_path}")
                    try:
                        from mcp_feedback_enhanced_desktop import (  # type: ignore
                            launch_desktop_app as dev_func,
                        )

                        return dev_func
                    except ImportError:
                        print("❌ 無法從開發環境路徑導入桌面應用程式模組")
                        return None
                else:
                    print(f"⚠️  開發環境路徑不存在: {tauri_python_path}")
                    print("💡 這可能是 PyPI 安裝的版本，桌面應用功能不可用")
                    return None

            launch_desktop_app_func = import_desktop_app()
            if launch_desktop_app_func is None:
                print("❌ 桌面應用程式不可用")
                print("💡 可能的原因：")
                print("   1. 此版本不包含桌面應用程式二進制檔案")
                print("   2. 請使用包含桌面應用的版本，或使用 Web 模式")
                print("   3. Web 模式指令：uvx mcp-feedback-enhanced test --web")
                return False

            print("✅ 桌面應用程式模組導入成功")

        except ImportError as e:
            print(f"❌ 無法導入桌面應用程式模組: {e}")
            print(
                "💡 請確保已執行 'make build-desktop' 或 'python scripts/build_desktop.py'"
            )
            return False

        print("🚀 啟動桌面應用程式...")

        # 設置桌面模式環境變數
        os.environ["MCP_DESKTOP_MODE"] = "true"

        # 使用 asyncio 啟動桌面應用程式
        if sys.platform == "win32":
            asyncio.set_event_loop_policy(asyncio.WindowsProactorEventLoopPolicy())

        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)

        try:
            # 使用 WebUIManager 來管理桌面應用實例
            from .web.main import get_web_ui_manager

            manager = get_web_ui_manager()

            # 啟動桌面應用並保存實例到 manager
            app = loop.run_until_complete(launch_desktop_app_func(test_mode=True))
            manager.desktop_app_instance = app

            print("✅ 桌面應用程式啟動成功")
            print("💡 桌面應用程式正在運行，按 Ctrl+C 停止...")

            # 保持應用程式運行
            try:
                while True:
                    import time

                    time.sleep(1)
            except KeyboardInterrupt:
                print("\n🛑 停止桌面應用程式...")
                app.stop()
                return True

        except Exception as e:
            print(f"❌ 桌面應用程式啟動失敗: {e}")
            import traceback

            traceback.print_exc()
            return False
        finally:
            loop.close()

    except Exception as e:
        print(f"❌ 桌面應用程式測試失敗: {e}")
        import traceback

        traceback.print_exc()
        return False
    finally:
        # 清理環境變數
        os.environ.pop("MCP_DESKTOP_MODE", None)


async def wait_for_process(process):
    """等待進程結束"""
    try:
        # 等待進程自然結束
        await process.wait()

        # 確保管道正確關閉
        try:
            if hasattr(process, "stdout") and process.stdout:
                process.stdout.close()
            if hasattr(process, "stderr") and process.stderr:
                process.stderr.close()
            if hasattr(process, "stdin") and process.stdin:
                process.stdin.close()
        except Exception as close_error:
            print(f"關閉進程管道時出錯: {close_error}")

    except Exception as e:
        print(f"等待進程時出錯: {e}")


def show_version():
    """顯示版本資訊"""
    from . import __author__, __version__

    print(f"MCP Feedback Enhanced Enhanced v{__version__}")
    print(f"作者: {__author__}")
    print("GitHub: https://github.com/Minidoracat/mcp-feedback-enhanced")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
壓縮性能監控工具
================

監控 Gzip 壓縮的性能效果，包括壓縮比率、響應時間和文件大小統計。
提供實時性能數據和優化建議。
"""

import threading
from dataclasses import dataclass, field
from datetime import datetime, timedelta


@dataclass
class CompressionMetrics:
    """壓縮指標數據類"""

    timestamp: datetime
    path: str
    original_size: int
    compressed_size: int
    compression_ratio: float
    response_time: float
    content_type: str
    was_compressed: bool


@dataclass
class CompressionSummary:
    """壓縮摘要統計"""

    total_requests: int = 0
    compressed_requests: int = 0
    total_original_bytes: int = 0
    total_compressed_bytes: int = 0
    average_compression_ratio: float = 0.0
    average_response_time: float = 0.0
    compression_percentage: float = 0.0
    bandwidth_saved: int = 0
    top_compressed_paths: list[tuple[str, float]] = field(default_factory=list)


class CompressionMonitor:
    """壓縮性能監控器"""

    def __init__(self, max_metrics: int = 1000):
        self.max_metrics = max_metrics
        self.metrics: list[CompressionMetrics] = []
        self.lock = threading.Lock()
        self._start_time = datetime.now()

        # 路徑統計
        self.path_stats: dict[str, dict] = {}

        # 內容類型統計
        self.content_type_stats: dict[str, dict] = {}

    def record_request(
        self,
        path: str,
        original_size: int,
        compressed_size: int,
        response_time: float,
        content_type: str = "",
        was_compressed: bool = False,
    ):
        """記錄請求的壓縮數據"""

        compression_ratio = 0.0
        if original_size > 0 and was_compressed:
            compression_ratio = (1 - compressed_size / original_size) * 100

        metric = CompressionMetrics(
            timestamp=datetime.now(),
            path=path,
            original_size=original_size,
            compressed_size=compressed_size,
            compression_ratio=compression_ratio,
            response_time=response_time,
            content_type=content_type,
            was_compressed=was_compressed,
        )

        with self.lock:
            self.metrics.append(metric)

            # 限制記錄數量
            if len(self.metrics) > self.max_metrics:
                self.metrics = self.metrics[-self.max_metrics :]

            # 更新路徑統計
            self._update_path_stats(metric)

            # 更新內容類型統計
            self._update_content_type_stats(metric)

    def _update_path_stats(self, metric: CompressionMetrics):
        """更新路徑統計"""
        path = metric.path
        if path not in self.path_stats:
            self.path_stats[path] = {
                "requests": 0,
                "compressed_requests": 0,
                "total_original_bytes": 0,
                "total_compressed_bytes": 0,
                "total_response_time": 0.0,
                "best_compression_ratio": 0.0,
            }

        stats = self.path_stats[path]
        stats["requests"] += 1
        stats["total_original_bytes"] += metric.original_size
        stats["total_compressed_bytes"] += metric.compressed_size
        stats["total_response_time"] += metric.response_time

        if metric.was_compressed:
            stats["compressed_requests"] += 1
            stats["best_compression_ratio"] = max(
                stats["best_compression_ratio"], metric.compression_ratio
            )

    def _update_content_type_stats(self, metric: CompressionMetrics):
        """更新內容類型統計"""
        content_type = metric.content_type or "unknown"
        if content_type not in self.content_type_stats:
            self.content_type_stats[content_type] = {
                "requests": 0,
                "compressed_requests": 0,
                "total_original_bytes": 0,
                "total_compressed_bytes": 0,
                "average_compression_ratio": 0.0,
            }

        stats = self.content_type_stats[content_type]
        stats["requests"] += 1
        stats["total_original_bytes"] += metric.original_size
        stats["total_compressed_bytes"] += metric.compressed_size

        if metric.was_compressed:
            stats["compressed_requests"] += 1

            # 重新計算平均壓縮比
            if stats["total_original_bytes"] > 0:
                stats["average_compression_ratio"] = (
                    1 - stats["total_compressed_bytes"] / stats["total_original_bytes"]
                ) * 100

    def get_summary(self, time_window: timedelta | None = None) -> CompressionSummary:
        """獲取壓縮摘要統計"""
        with self.lock:
            metrics = self.metrics

            # 如果指定時間窗口，過濾數據
            if time_window:
                cutoff_time = datetime.now() - time_window
                metrics = [m for m in metrics if m.timestamp >= cutoff_time]

            if not metrics:
                return CompressionSummary()

            total_requests = len(metrics)
            compressed_requests = sum(1 for m in metrics if m.was_compressed)
            total_original_bytes = sum(m.original_size for m in metrics)
            total_compressed_bytes = sum(m.compressed_size for m in metrics)
            total_response_time = sum(m.response_time for m in metrics)

            # 計算統計數據
            compression_percentage = (
                (compressed_requests / total_requests * 100)
                if total_requests > 0
                else 0
            )
            average_compression_ratio = 0.0
            bandwidth_saved = 0

            if total_original_bytes > 0:
                average_compression_ratio = (
                    1 - total_compressed_bytes / total_original_bytes
                ) * 100
                bandwidth_saved = total_original_bytes - total_compressed_bytes

            average_response_time = (
                total_response_time / total_requests if total_requests > 0 else 0
            )

            # 獲取壓縮效果最好的路徑
            top_compressed_paths = self._get_top_compressed_paths()

            return CompressionSummary(
                total_requests=total_requests,
                compressed_requests=compressed_requests,
                total_original_bytes=total_original_bytes,
                total_compressed_bytes=total_compressed_bytes,
                average_compression_ratio=average_compression_ratio,
                average_response_time=average_response_time,
                compression_percentage=compression_percentage,
                bandwidth_saved=bandwidth_saved,
                top_compressed_paths=top_compressed_paths,
            )

    def _get_top_compressed_paths(self, limit: int = 5) -> list[tuple[str, float]]:
        """獲取壓縮效果最好的路徑"""
        path_ratios = []

        for path, stats in self.path_stats.items():
            if stats["compressed_requests"] > 0 and stats["total_original_bytes"] > 0:
                compression_ratio = (
                    1 - stats["total_compressed_bytes"] / stats["total_original_bytes"]
                ) * 100
                path_ratios.append((path, compression_ratio))

        # 按壓縮比排序
        path_ratios.sort(key=lambda x: x[1], reverse=True)
        return path_ratios[:limit]

    def get_path_stats(self) -> dict[str, dict]:
        """獲取路徑統計"""
        with self.lock:
            return self.path_stats.copy()

    def get_content_type_stats(self) -> dict[str, dict]:
        """獲取內容類型統計"""
        with self.lock:
            return self.content_type_stats.copy()

    def get_recent_metrics(self, limit: int = 100) -> list[CompressionMetrics]:
        """獲取最近的指標數據"""
        with self.lock:
            return self.metrics[-limit:] if self.metrics else []

    def reset_stats(self):
        """重置統計數據"""
        with self.lock:
            self.metrics.clear()
            self.path_stats.clear()
            self.content_type_stats.clear()
            self._start_time = datetime.now()

    def export_stats(self) -> dict:
        """導出統計數據為字典格式"""
        summary = self.get_summary()

        return {
            "summary": {
                "total_requests": summary.total_requests,
                "compressed_requests": summary.compressed_requests,
                "compression_percentage": round(summary.compression_percentage, 2),
                "average_compression_ratio": round(
                    summary.average_compression_ratio, 2
                ),
                "bandwidth_saved_mb": round(summary.bandwidth_saved / (1024 * 1024), 2),
                "average_response_time_ms": round(
                    summary.average_response_time * 1000, 2
                ),
                "monitoring_duration_hours": round(
                    (datetime.now() - self._start_time).total_seconds() / 3600, 2
                ),
            },
            "top_compressed_paths": [
                {"path": path, "compression_ratio": round(ratio, 2)}
                for path, ratio in summary.top_compressed_paths
            ],
            "path_stats": {
                path: {
                    "requests": stats["requests"],
                    "compression_percentage": round(
                        stats["compressed_requests"] / stats["requests"] * 100, 2
                    )
                    if stats["requests"] > 0
                    else 0,
                    "average_response_time_ms": round(
                        stats["total_response_time"] / stats["requests"] * 1000, 2
                    )
                    if stats["requests"] > 0
                    else 0,
                    "bandwidth_saved_kb": round(
                        (
                            stats["total_original_bytes"]
                            - stats["total_compressed_bytes"]
                        )
                        / 1024,
                        2,
                    ),
                }
                for path, stats in self.path_stats.items()
            },
            "content_type_stats": {
                content_type: {
                    "requests": stats["requests"],
                    "compression_percentage": round(
                        stats["compressed_requests"] / stats["requests"] * 100, 2
                    )
                    if stats["requests"] > 0
                    else 0,
                    "average_compression_ratio": round(
                        stats["average_compression_ratio"], 2
                    ),
                }
                for content_type, stats in self.content_type_stats.items()
            },
        }


# 全域監控器實例
_compression_monitor: CompressionMonitor | None = None


def get_compression_monitor() -> CompressionMonitor:
    """獲取全域壓縮監控器實例"""
    global _compression_monitor
    if _compression_monitor is None:
        _compression_monitor = CompressionMonitor()
    return _compression_monitor



#!/usr/bin/env python3
"""
測試配置和共用 fixtures
"""

import asyncio
import os
import shutil
import tempfile
from collections.abc import Generator
from pathlib import Path
from typing import Any

import pytest

# 使用正確的模組導入，不手動修改 sys.path
from mcp_feedback_enhanced.i18n import get_i18n_manager
from mcp_feedback_enhanced.web.main import WebUIManager


@pytest.fixture(scope="session")
def event_loop():
    """創建事件循環 fixture"""
    loop = asyncio.new_event_loop()
    yield loop
    loop.close()


@pytest.fixture
def temp_dir() -> Generator[Path, None, None]:
    """創建臨時目錄 fixture"""
    temp_path = Path(tempfile.mkdtemp())
    yield temp_path
    shutil.rmtree(temp_path, ignore_errors=True)


@pytest.fixture
def test_project_dir(temp_dir: Path) -> Path:
    """創建測試專案目錄"""
    project_dir = temp_dir / "test_project"
    project_dir.mkdir()

    # 創建一些測試文件
    (project_dir / "README.md").write_text("# Test Project")
    (project_dir / "main.py").write_text("print('Hello World')")

    return project_dir


@pytest.fixture
def web_ui_manager() -> Generator[WebUIManager, None, None]:
    """創建 WebUIManager fixture"""
    import os

    # 設置測試模式環境變數
    original_test_mode = os.environ.get("MCP_TEST_MODE")
    original_web_port = os.environ.get("MCP_WEB_PORT")

    os.environ["MCP_TEST_MODE"] = "true"
    # 使用動態端口範圍避免衝突
    os.environ["MCP_WEB_PORT"] = "0"  # 讓系統自動分配端口

    try:
        manager = WebUIManager(host="127.0.0.1")  # 使用環境變數控制端口
        yield manager
    finally:
        # 恢復原始環境變數
        if original_test_mode is not None:
            os.environ["MCP_TEST_MODE"] = original_test_mode
        else:
            os.environ.pop("MCP_TEST_MODE", None)

        if original_web_port is not None:
            os.environ["MCP_WEB_PORT"] = original_web_port
        else:
            os.environ.pop("MCP_WEB_PORT", None)

        # 清理
        if manager.server_thread and manager.server_thread.is_alive():
            # 這裡可以添加服務器停止邏輯
            pass


@pytest.fixture
def i18n_manager():
    """創建 I18N 管理器 fixture"""
    return get_i18n_manager()


@pytest.fixture
def test_config() -> dict[str, Any]:
    """測試配置 fixture"""
    return {
        "timeout": 30,
        "debug": True,
        "web_port": 8765,
        "test_summary": "測試摘要 - 這是一個自動化測試",
        "test_feedback": "這是測試回饋內容",
    }


@pytest.fixture(autouse=True)
def setup_test_env():
    """自動設置測試環境"""
    # 設置測試環境變數
    original_debug = os.environ.get("MCP_DEBUG")
    os.environ["MCP_DEBUG"] = "true"

    yield

    # 恢復原始環境
    if original_debug is not None:
        os.environ["MCP_DEBUG"] = original_debug
    else:
        os.environ.pop("MCP_DEBUG", None)



#!/usr/bin/env python3
"""
國際化支援模組
===============

提供統一的多語系支援功能，支援繁體中文、英文等語言。
自動偵測系統語言，並提供語言切換功能。

新架構：
- 使用分離的 JSON 翻譯檔案
- 支援巢狀翻譯鍵值
- 元資料支援
- 易於擴充新語言

作者: Minidoracat
"""

import json
import locale
import os
from pathlib import Path
from typing import Any

from .debug import i18n_debug_log as debug_log


class I18nManager:
    """國際化管理器 - 新架構版本"""

    def __init__(self):
        self._current_language = None
        self._translations = {}
        self._supported_languages = ["zh-TW", "en", "zh-CN"]
        self._fallback_language = "en"
        self._config_file = self._get_config_file_path()
        self._locales_dir = Path(__file__).parent / "web" / "locales"

        # 載入翻譯
        self._load_all_translations()

        # 設定語言
        self._current_language = self._detect_language()

    def _get_config_file_path(self) -> Path:
        """獲取配置文件路徑"""
        config_dir = Path.home() / ".config" / "mcp-feedback-enhanced"
        config_dir.mkdir(parents=True, exist_ok=True)
        return config_dir / "language.json"

    def _load_all_translations(self) -> None:
        """載入所有語言的翻譯檔案"""
        self._translations = {}

        for lang_code in self._supported_languages:
            lang_dir = self._locales_dir / lang_code
            translation_file = lang_dir / "translation.json"

            if translation_file.exists():
                try:
                    with open(translation_file, encoding="utf-8") as f:
                        data = json.load(f)
                        self._translations[lang_code] = data
                        debug_log(
                            f"成功載入語言 {lang_code}: {data.get('meta', {}).get('displayName', lang_code)}"
                        )
                except Exception as e:
                    debug_log(f"載入語言檔案失敗 {lang_code}: {e}")
                    # 如果載入失敗，使用空的翻譯
                    self._translations[lang_code] = {}
            else:
                debug_log(f"找不到語言檔案: {translation_file}")
                self._translations[lang_code] = {}

    def _detect_language(self) -> str:
        """自動偵測語言"""
        # 1. 優先使用用戶保存的語言設定
        saved_lang = self._load_saved_language()
        if saved_lang and saved_lang in self._supported_languages:
            return saved_lang

        # 2. 檢查環境變數
        env_lang = os.getenv("MCP_LANGUAGE", "").strip()
        if env_lang and env_lang in self._supported_languages:
            return env_lang

        # 3. 檢查其他環境變數（LANG, LC_ALL 等）
        for env_var in ["LANG", "LC_ALL", "LC_MESSAGES", "LANGUAGE"]:
            env_value = os.getenv(env_var, "").strip()
            if env_value:
                if env_value.startswith("zh_TW") or env_value.startswith("zh_Hant"):
                    return "zh-TW"
                if env_value.startswith("zh_CN") or env_value.startswith("zh_Hans"):
                    return "zh-CN"
                if env_value.startswith("en"):
                    return "en"

        # 4. 自動偵測系統語言（僅在非測試模式下）
        if not os.getenv("MCP_TEST_MODE"):
            try:
                # 獲取系統語言
                system_locale = locale.getdefaultlocale()[0]
                if system_locale:
                    if system_locale.startswith("zh_TW") or system_locale.startswith(
                        "zh_Hant"
                    ):
                        return "zh-TW"
                    if system_locale.startswith("zh_CN") or system_locale.startswith(
                        "zh_Hans"
                    ):
                        return "zh-CN"
                    if system_locale.startswith("en"):
                        return "en"
            except Exception:
                pass

        # 5. 回退到默認語言
        return self._fallback_language

    def _load_saved_language(self) -> str | None:
        """載入保存的語言設定"""
        try:
            if self._config_file.exists():
                with open(self._config_file, encoding="utf-8") as f:
                    config = json.load(f)
                    language = config.get("language")
                    return language if isinstance(language, str) else None
        except Exception:
            pass
        return None

    def save_language(self, language: str) -> None:
        """保存語言設定"""
        try:
            config = {"language": language}
            with open(self._config_file, "w", encoding="utf-8") as f:
                json.dump(config, f, ensure_ascii=False, indent=2)
        except Exception:
            pass

    def get_current_language(self) -> str:
        """獲取當前語言"""
        return self._current_language or "zh-TW"

    def set_language(self, language: str) -> bool:
        """設定語言"""
        if language in self._supported_languages:
            self._current_language = language
            self.save_language(language)
            return True
        return False

    def get_supported_languages(self) -> list[str]:
        """獲取支援的語言列表"""
        return self._supported_languages.copy()

    def get_language_info(self, language_code: str) -> dict[str, Any]:
        """獲取語言的元資料信息"""
        if language_code in self._translations:
            meta = self._translations[language_code].get("meta", {})
            return meta if isinstance(meta, dict) else {}
        return {}

    def _get_nested_value(self, data: dict[str, Any], key_path: str) -> str | None:
        """從巢狀字典中獲取值，支援點分隔的鍵路徑"""
        keys = key_path.split(".")
        current: Any = data

        for key in keys:
            if isinstance(current, dict) and key in current:
                current = current[key]
            else:
                return None

        return str(current) if isinstance(current, str) else None

    def t(self, key: str, **kwargs) -> str:
        """
        翻譯函數 - 支援新舊兩種鍵值格式

        新格式: 'buttons.submit' -> data['buttons']['submit']
        舊格式: 'btn_submit_feedback' -> 兼容舊的鍵值
        """
        # 獲取當前語言的翻譯
        current_translations = self._translations.get(self._current_language, {})

        # 嘗試新格式（巢狀鍵）
        text = self._get_nested_value(current_translations, key)

        # 如果沒有找到，嘗試舊格式的兼容映射
        if text is None:
            text = self._get_legacy_translation(current_translations, key)

        # 如果還是沒有找到，嘗試使用回退語言
        if text is None:
            fallback_translations = self._translations.get(self._fallback_language, {})
            text = self._get_nested_value(fallback_translations, key)
            if text is None:
                text = self._get_legacy_translation(fallback_translations, key)

        # 最後回退到鍵本身
        if text is None:
            text = key

        # 處理格式化參數
        if kwargs:
            try:
                text = text.format(**kwargs)
            except (KeyError, ValueError):
                pass

        return text

    def _get_legacy_translation(
        self, translations: dict[str, Any], key: str
    ) -> str | None:
        """獲取舊格式翻譯的兼容方法"""
        # 舊鍵到新鍵的映射
        legacy_mapping = {
            # 應用程式
            "app_title": "app.title",
            "project_directory": "app.projectDirectory",
            "language": "app.language",
            "settings": "app.settings",
            # 分頁
            "feedback_tab": "tabs.feedback",
            "command_tab": "tabs.command",
            "images_tab": "tabs.images",
            # 回饋
            "feedback_title": "feedback.title",
            "feedback_description": "feedback.description",
            "feedback_placeholder": "feedback.placeholder",
            # 命令
            "command_title": "command.title",
            "command_description": "command.description",
            "command_placeholder": "command.placeholder",
            "command_output": "command.output",
            # 圖片
            "images_title": "images.title",
            "images_select": "images.select",
            "images_paste": "images.paste",
            "images_clear": "images.clear",
            "images_status": "images.status",
            "images_status_with_size": "images.statusWithSize",
            "images_drag_hint": "images.dragHint",
            "images_delete_confirm": "images.deleteConfirm",
            "images_delete_title": "images.deleteTitle",
            "images_size_warning": "images.sizeWarning",
            "images_format_error": "images.formatError",
            # 按鈕
            "submit": "buttons.submit",
            "cancel": "buttons.cancel",
            "close": "buttons.close",
            "clear": "buttons.clear",
            "btn_submit_feedback": "buttons.submitFeedback",
            "btn_cancel": "buttons.cancel",
            "btn_select_files": "buttons.selectFiles",
            "btn_paste_clipboard": "buttons.pasteClipboard",
            "btn_clear_all": "buttons.clearAll",
            "btn_run_command": "buttons.runCommand",
            # 狀態
            "feedback_submitted": "status.feedbackSubmitted",
            "feedback_cancelled": "status.feedbackCancelled",
            "timeout_message": "status.timeoutMessage",
            "error_occurred": "status.errorOccurred",
            "loading": "status.loading",
            "connecting": "status.connecting",
            "connected": "status.connected",
            "disconnected": "status.disconnected",
            "uploading": "status.uploading",
            "upload_success": "status.uploadSuccess",
            "upload_failed": "status.uploadFailed",
            "command_running": "status.commandRunning",
            "command_finished": "status.commandFinished",
            "paste_success": "status.pasteSuccess",
            "paste_failed": "status.pasteFailed",
            "invalid_file_type": "status.invalidFileType",
            "file_too_large": "status.fileTooLarge",
            # 其他
            "ai_summary": "aiSummary",
            "language_selector": "languageSelector",
            "language_zh_tw": "languageNames.zhTw",
            "language_en": "languageNames.en",
            "language_zh_cn": "languageNames.zhCn",
            # 測試
            "test_web_ui_summary": "test.webUiSummary",
        }

        # 檢查是否有對應的新鍵
        new_key = legacy_mapping.get(key)
        if new_key:
            return self._get_nested_value(translations, new_key)

        return None

    def get_language_display_name(self, language_code: str) -> str:
        """獲取語言的顯示名稱"""
        # 直接從當前語言的翻譯中獲取，避免遞歸
        current_translations = self._translations.get(self._current_language, {})

        # 根據語言代碼構建鍵值
        lang_key = None
        if language_code == "zh-TW":
            lang_key = "languageNames.zhTw"
        elif language_code == "zh-CN":
            lang_key = "languageNames.zhCn"
        elif language_code == "en":
            lang_key = "languageNames.en"
        else:
            # 通用格式
            lang_key = f"languageNames.{language_code.replace('-', '').lower()}"

        # 直接獲取翻譯，避免調用 self.t() 產生遞歸
        if lang_key:
            display_name = self._get_nested_value(current_translations, lang_key)
            if display_name:
                return display_name

        # 回退到元資料中的顯示名稱
        meta = self.get_language_info(language_code)
        display_name = meta.get("displayName", language_code)
        return str(display_name) if display_name else language_code

    def reload_translations(self) -> None:
        """重新載入所有翻譯檔案（開發時使用）"""
        self._load_all_translations()

    def add_language(self, language_code: str, translation_file_path: str) -> bool:
        """動態添加新語言支援"""
        try:
            translation_file = Path(translation_file_path)
            if not translation_file.exists():
                return False

            with open(translation_file, encoding="utf-8") as f:
                data = json.load(f)
                self._translations[language_code] = data

                if language_code not in self._supported_languages:
                    self._supported_languages.append(language_code)

                debug_log(
                    f"成功添加語言 {language_code}: {data.get('meta', {}).get('displayName', language_code)}"
                )
                return True
        except Exception as e:
            debug_log(f"添加語言失敗 {language_code}: {e}")
            return False


# 全域的國際化管理器實例
_i18n_manager = None


def get_i18n_manager() -> I18nManager:
    """獲取全域的國際化管理器實例"""
    global _i18n_manager
    if _i18n_manager is None:
        _i18n_manager = I18nManager()
    return _i18n_manager


def t(key: str, **kwargs) -> str:
    """便捷的翻譯函數"""
    return get_i18n_manager().t(key, **kwargs)


def set_language(language: str) -> bool:
    """設定語言"""
    return get_i18n_manager().set_language(language)


def get_current_language() -> str:
    """獲取當前語言"""
    return get_i18n_manager().get_current_language()


def reload_translations() -> None:
    """重新載入翻譯（開發用）"""
    get_i18n_manager().reload_translations()



#!/usr/bin/env python3
"""
GitHub Actions 工作流程驗證腳本

此腳本驗證 GitHub Actions 工作流程文件的語法和配置正確性。
"""

import sys
from pathlib import Path

import yaml


def validate_yaml_syntax(file_path: Path) -> bool:
    """驗證 YAML 文件語法"""
    try:
        with open(file_path, encoding="utf-8") as f:
            yaml.safe_load(f)
        print(f"✅ {file_path.name}: YAML 語法正確")
        return True
    except yaml.YAMLError as e:
        print(f"❌ {file_path.name}: YAML 語法錯誤 - {e}")
        return False
    except Exception as e:
        print(f"❌ {file_path.name}: 讀取文件失敗 - {e}")
        return False


def validate_workflow_structure(file_path: Path) -> bool:
    """驗證工作流程結構"""
    try:
        with open(file_path, encoding="utf-8") as f:
            workflow = yaml.safe_load(f)

        # 檢查是否成功解析
        if workflow is None:
            print(f"❌ {file_path.name}: 文件為空或解析失敗")
            return False

        # 檢查必需的頂級字段
        # 注意：YAML 會將 'on' 解析為 True，所以我們需要特殊處理
        required_fields = ["name", "jobs"]
        for field in required_fields:
            if field not in workflow:
                print(f"❌ {file_path.name}: 缺少必需字段 '{field}'")
                print(f"   實際字段: {list(workflow.keys())}")
                return False

        # 檢查 'on' 字段（可能被解析為 True）
        if "on" not in workflow and True not in workflow:
            print(f"❌ {file_path.name}: 缺少觸發條件 'on'")
            print(f"   實際字段: {list(workflow.keys())}")
            return False

        # 檢查 jobs 結構
        if not isinstance(workflow["jobs"], dict):
            print(f"❌ {file_path.name}: 'jobs' 必須是字典")
            return False

        if not workflow["jobs"]:
            print(f"❌ {file_path.name}: 'jobs' 不能為空")
            return False

        print(f"✅ {file_path.name}: 工作流程結構正確")
        return True

    except Exception as e:
        print(f"❌ {file_path.name}: 結構驗證失敗 - {e}")
        return False


def validate_build_desktop_workflow(file_path: Path) -> bool:
    """驗證桌面構建工作流程的特定配置"""
    try:
        with open(file_path, encoding="utf-8") as f:
            workflow = yaml.safe_load(f)

        # 檢查 matrix 配置
        build_job = workflow["jobs"].get("build-desktop", {})
        strategy = build_job.get("strategy", {})
        matrix = strategy.get("matrix", {})

        if "include" not in matrix:
            print(f"❌ {file_path.name}: 缺少 matrix.include 配置")
            return False

        # 檢查平台配置
        platforms = matrix["include"]
        expected_platforms = {"windows", "macos-intel", "macos-arm64", "linux"}
        actual_platforms = {item.get("name") for item in platforms}

        if not expected_platforms.issubset(actual_platforms):
            missing = expected_platforms - actual_platforms
            print(f"❌ {file_path.name}: 缺少平台配置: {missing}")
            return False

        print(f"✅ {file_path.name}: 桌面構建配置正確")
        return True

    except Exception as e:
        print(f"❌ {file_path.name}: 桌面構建驗證失敗 - {e}")
        return False


def validate_publish_workflow(file_path: Path) -> bool:
    """驗證發佈工作流程的特定配置"""
    try:
        with open(file_path, encoding="utf-8") as f:
            workflow = yaml.safe_load(f)

        # 檢查輸入參數 - 注意 'on' 可能被解析為 True
        on_section = workflow.get("on") or workflow.get(True)
        if not on_section:
            print(f"❌ {file_path.name}: 找不到觸發條件")
            return False

        workflow_dispatch = on_section.get("workflow_dispatch", {})
        inputs = workflow_dispatch.get("inputs", {})

        required_inputs = {"version_type", "include_desktop"}
        actual_inputs = set(inputs.keys())

        if not required_inputs.issubset(actual_inputs):
            missing = required_inputs - actual_inputs
            print(f"❌ {file_path.name}: 缺少輸入參數: {missing}")
            print(f"   實際輸入參數: {actual_inputs}")
            return False

        # 檢查是否有桌面應用處理步驟
        release_job = workflow["jobs"].get("release", {})
        steps = release_job.get("steps", [])

        has_desktop_steps = any(
            "desktop" in step.get("name", "").lower() for step in steps
        )

        if not has_desktop_steps:
            print(f"❌ {file_path.name}: 缺少桌面應用處理步驟")
            return False

        print(f"✅ {file_path.name}: 發佈工作流程配置正確")
        return True

    except Exception as e:
        print(f"❌ {file_path.name}: 發佈工作流程驗證失敗 - {e}")
        return False


def main():
    """主函數"""
    print("🔍 驗證 GitHub Actions 工作流程...")
    print()

    # 獲取工作流程目錄
    workflows_dir = Path(__file__).parent.parent / ".github" / "workflows"

    if not workflows_dir.exists():
        print(f"❌ 工作流程目錄不存在: {workflows_dir}")
        sys.exit(1)

    # 查找所有工作流程文件
    workflow_files = list(workflows_dir.glob("*.yml")) + list(
        workflows_dir.glob("*.yaml")
    )

    if not workflow_files:
        print(f"❌ 在 {workflows_dir} 中沒有找到工作流程文件")
        sys.exit(1)

    print(f"📁 找到 {len(workflow_files)} 個工作流程文件")
    print()

    # 驗證每個文件
    all_valid = True

    for workflow_file in sorted(workflow_files):
        print(f"🔍 驗證 {workflow_file.name}...")

        # 基本語法驗證
        if not validate_yaml_syntax(workflow_file):
            all_valid = False
            continue

        # 結構驗證
        if not validate_workflow_structure(workflow_file):
            all_valid = False
            continue

        # 特定工作流程驗證
        if workflow_file.name == "build-desktop.yml":
            if not validate_build_desktop_workflow(workflow_file):
                all_valid = False
        elif workflow_file.name == "publish.yml":
            if not validate_publish_workflow(workflow_file):
                all_valid = False

        print()

    # 總結
    if all_valid:
        print("🎉 所有工作流程文件驗證通過！")
        print()
        print("📋 下一步:")
        print("  1. 提交並推送更改到 GitHub")
        print("  2. 測試 'Build Desktop Applications' 工作流程")
        print("  3. 測試 'Build Desktop & Release' 工作流程")
        print("  4. 驗證桌面應用是否正確包含在發佈中")
    else:
        print("❌ 部分工作流程文件驗證失敗")
        print("請修復上述問題後重新運行驗證")
        sys.exit(1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
主要路由處理
============

設置 Web UI 的主要路由和處理邏輯。
"""

import json
import time
from pathlib import Path
from typing import TYPE_CHECKING

from fastapi import Request, WebSocket, WebSocketDisconnect
from fastapi.responses import HTMLResponse, JSONResponse

from ... import __version__
from ...debug import web_debug_log as debug_log


if TYPE_CHECKING:
    from ..main import WebUIManager


def load_user_layout_settings() -> str:
    """載入用戶的佈局模式設定"""
    try:
        # 使用統一的設定檔案路徑
        config_dir = Path.home() / ".config" / "mcp-feedback-enhanced"
        settings_file = config_dir / "ui_settings.json"

        if settings_file.exists():
            with open(settings_file, encoding="utf-8") as f:
                settings = json.load(f)
                layout_mode = settings.get("layoutMode", "combined-vertical")
                debug_log(f"從設定檔案載入佈局模式: {layout_mode}")
                # 修復 no-any-return 錯誤 - 確保返回 str 類型
                return str(layout_mode)
        else:
            debug_log("設定檔案不存在，使用預設佈局模式: combined-vertical")
            return "combined-vertical"
    except Exception as e:
        debug_log(f"載入佈局設定失敗: {e}，使用預設佈局模式: combined-vertical")
        return "combined-vertical"


def setup_routes(manager: "WebUIManager"):
    """設置路由"""

    @manager.app.get("/", response_class=HTMLResponse)
    async def index(request: Request):
        """統一回饋頁面 - 重構後的主頁面"""
        # 獲取當前活躍會話
        current_session = manager.get_current_session()

        if not current_session:
            # 沒有活躍會話時顯示等待頁面
            return manager.templates.TemplateResponse(
                "index.html",
                {
                    "request": request,
                    "title": "MCP Feedback Enhanced",
                    "has_session": False,
                    "version": __version__,
                },
            )

        # 有活躍會話時顯示回饋頁面
        # 載入用戶的佈局模式設定
        layout_mode = load_user_layout_settings()

        return manager.templates.TemplateResponse(
            "feedback.html",
            {
                "request": request,
                "project_directory": current_session.project_directory,
                "summary": current_session.summary,
                "title": "Interactive Feedback - 回饋收集",
                "version": __version__,
                "has_session": True,
                "layout_mode": layout_mode,
                "i18n": manager.i18n,
            },
        )

    @manager.app.get("/api/translations")
    async def get_translations():
        """獲取翻譯數據 - 從 Web 專用翻譯檔案載入"""
        translations = {}

        # 獲取 Web 翻譯檔案目錄
        web_locales_dir = Path(__file__).parent.parent / "locales"
        supported_languages = ["zh-TW", "zh-CN", "en"]

        for lang_code in supported_languages:
            lang_dir = web_locales_dir / lang_code
            translation_file = lang_dir / "translation.json"

            try:
                if translation_file.exists():
                    with open(translation_file, encoding="utf-8") as f:
                        lang_data = json.load(f)
                        translations[lang_code] = lang_data
                        debug_log(f"成功載入 Web 翻譯: {lang_code}")
                else:
                    debug_log(f"Web 翻譯檔案不存在: {translation_file}")
                    translations[lang_code] = {}
            except Exception as e:
                debug_log(f"載入 Web 翻譯檔案失敗 {lang_code}: {e}")
                translations[lang_code] = {}

        debug_log(f"Web 翻譯 API 返回 {len(translations)} 種語言的數據")
        return JSONResponse(content=translations)

    @manager.app.get("/api/session-status")
    async def get_session_status():
        """獲取當前會話狀態"""
        current_session = manager.get_current_session()

        if not current_session:
            return JSONResponse(
                content={
                    "has_session": False,
                    "status": "no_session",
                    "message": "沒有活躍會話",
                }
            )

        return JSONResponse(
            content={
                "has_session": True,
                "status": "active",
                "session_info": {
                    "project_directory": current_session.project_directory,
                    "summary": current_session.summary,
                    "feedback_completed": current_session.feedback_completed.is_set(),
                },
            }
        )

    @manager.app.get("/api/current-session")
    async def get_current_session():
        """獲取當前會話詳細信息"""
        current_session = manager.get_current_session()

        if not current_session:
            return JSONResponse(status_code=404, content={"error": "沒有活躍會話"})

        return JSONResponse(
            content={
                "session_id": current_session.session_id,
                "project_directory": current_session.project_directory,
                "summary": current_session.summary,
                "feedback_completed": current_session.feedback_completed.is_set(),
                "command_logs": current_session.command_logs,
                "images_count": len(current_session.images),
            }
        )

    @manager.app.websocket("/ws")
    async def websocket_endpoint(websocket: WebSocket):
        """WebSocket 端點 - 重構後移除 session_id 依賴"""
        # 獲取當前活躍會話
        session = manager.get_current_session()
        if not session:
            await websocket.close(code=4004, reason="沒有活躍會話")
            return

        await websocket.accept()

        # 檢查會話是否已有 WebSocket 連接
        if session.websocket and session.websocket != websocket:
            debug_log("會話已有 WebSocket 連接，替換為新連接")

        session.websocket = websocket
        debug_log(f"WebSocket 連接建立: 當前活躍會話 {session.session_id}")

        # 發送連接成功消息
        try:
            await websocket.send_json(
                {"type": "connection_established", "message": "WebSocket 連接已建立"}
            )

            # 檢查是否有待發送的會話更新
            if getattr(manager, "_pending_session_update", False):
                debug_log("檢測到待發送的會話更新，準備發送通知")
                await websocket.send_json(
                    {
                        "type": "session_updated",
                        "message": "新會話已創建，正在更新頁面內容",
                        "session_info": {
                            "project_directory": session.project_directory,
                            "summary": session.summary,
                            "session_id": session.session_id,
                        },
                    }
                )
                manager._pending_session_update = False
                debug_log("✅ 已發送會話更新通知到前端")
            else:
                # 發送當前會話狀態
                await websocket.send_json(
                    {"type": "status_update", "status_info": session.get_status_info()}
                )
                debug_log("已發送當前會話狀態到前端")

        except Exception as e:
            debug_log(f"發送連接確認失敗: {e}")

        try:
            while True:
                data = await websocket.receive_text()
                message = json.loads(data)

                # 重新獲取當前會話，以防會話已切換
                current_session = manager.get_current_session()
                if current_session and current_session.websocket == websocket:
                    await handle_websocket_message(manager, current_session, message)
                else:
                    debug_log("會話已切換或 WebSocket 連接不匹配，忽略消息")
                    break

        except WebSocketDisconnect:
            debug_log("WebSocket 連接正常斷開")
        except ConnectionResetError:
            debug_log("WebSocket 連接被重置")
        except Exception as e:
            debug_log(f"WebSocket 錯誤: {e}")
        finally:
            # 安全清理 WebSocket 連接
            current_session = manager.get_current_session()
            if current_session and current_session.websocket == websocket:
                current_session.websocket = None
                debug_log("已清理會話中的 WebSocket 連接")

    @manager.app.post("/api/save-settings")
    async def save_settings(request: Request):
        """保存設定到檔案"""
        try:
            data = await request.json()

            # 使用統一的設定檔案路徑
            config_dir = Path.home() / ".config" / "mcp-feedback-enhanced"
            config_dir.mkdir(parents=True, exist_ok=True)
            settings_file = config_dir / "ui_settings.json"

            # 保存設定到檔案
            with open(settings_file, "w", encoding="utf-8") as f:
                json.dump(data, f, ensure_ascii=False, indent=2)

            debug_log(f"設定已保存到: {settings_file}")

            return JSONResponse(content={"status": "success", "message": "設定已保存"})

        except Exception as e:
            debug_log(f"保存設定失敗: {e}")
            return JSONResponse(
                status_code=500,
                content={"status": "error", "message": f"保存失敗: {e!s}"},
            )

    @manager.app.get("/api/load-settings")
    async def load_settings():
        """從檔案載入設定"""
        try:
            # 使用統一的設定檔案路徑
            config_dir = Path.home() / ".config" / "mcp-feedback-enhanced"
            settings_file = config_dir / "ui_settings.json"

            if settings_file.exists():
                with open(settings_file, encoding="utf-8") as f:
                    settings = json.load(f)

                debug_log(f"設定已從檔案載入: {settings_file}")
                return JSONResponse(content=settings)
            debug_log("設定檔案不存在，返回空設定")
            return JSONResponse(content={})

        except Exception as e:
            debug_log(f"載入設定失敗: {e}")
            return JSONResponse(
                status_code=500,
                content={"status": "error", "message": f"載入失敗: {e!s}"},
            )

    @manager.app.post("/api/clear-settings")
    async def clear_settings():
        """清除設定檔案"""
        try:
            # 使用統一的設定檔案路徑
            config_dir = Path.home() / ".config" / "mcp-feedback-enhanced"
            settings_file = config_dir / "ui_settings.json"

            if settings_file.exists():
                settings_file.unlink()
                debug_log(f"設定檔案已刪除: {settings_file}")
            else:
                debug_log("設定檔案不存在，無需刪除")

            return JSONResponse(content={"status": "success", "message": "設定已清除"})

        except Exception as e:
            debug_log(f"清除設定失敗: {e}")
            return JSONResponse(
                status_code=500,
                content={"status": "error", "message": f"清除失敗: {e!s}"},
            )

    @manager.app.get("/api/load-session-history")
    async def load_session_history():
        """從檔案載入會話歷史"""
        try:
            # 使用統一的設定檔案路徑
            config_dir = Path.home() / ".config" / "mcp-feedback-enhanced"
            history_file = config_dir / "session_history.json"

            if history_file.exists():
                with open(history_file, encoding="utf-8") as f:
                    history_data = json.load(f)

                debug_log(f"會話歷史已從檔案載入: {history_file}")

                # 確保資料格式相容性
                if isinstance(history_data, dict):
                    # 新格式：包含版本資訊和其他元資料
                    sessions = history_data.get("sessions", [])
                    last_cleanup = history_data.get("lastCleanup", 0)
                else:
                    # 舊格式：直接是會話陣列（向後相容）
                    sessions = history_data if isinstance(history_data, list) else []
                    last_cleanup = 0

                # 回傳與 localStorage 格式相容的資料
                return JSONResponse(
                    content={"sessions": sessions, "lastCleanup": last_cleanup}
                )

            debug_log("會話歷史檔案不存在，返回空歷史")
            return JSONResponse(content={"sessions": [], "lastCleanup": 0})

        except Exception as e:
            debug_log(f"載入會話歷史失敗: {e}")
            return JSONResponse(
                status_code=500,
                content={"status": "error", "message": f"載入失敗: {e!s}"},
            )

    @manager.app.post("/api/save-session-history")
    async def save_session_history(request: Request):
        """保存會話歷史到檔案"""
        try:
            data = await request.json()

            # 使用統一的設定檔案路徑
            config_dir = Path.home() / ".config" / "mcp-feedback-enhanced"
            config_dir.mkdir(parents=True, exist_ok=True)
            history_file = config_dir / "session_history.json"

            # 建立新格式的資料結構
            history_data = {
                "version": "1.0",
                "sessions": data.get("sessions", []),
                "lastCleanup": data.get("lastCleanup", 0),
                "savedAt": int(time.time() * 1000),  # 當前時間戳
            }

            # 如果是首次儲存且有 localStorage 遷移標記
            if not history_file.exists() and data.get("migratedFrom") == "localStorage":
                history_data["migratedFrom"] = "localStorage"
                history_data["migratedAt"] = int(time.time() * 1000)

            # 保存會話歷史到檔案
            with open(history_file, "w", encoding="utf-8") as f:
                json.dump(history_data, f, ensure_ascii=False, indent=2)

            debug_log(f"會話歷史已保存到: {history_file}")
            session_count = len(history_data["sessions"])
            debug_log(f"保存了 {session_count} 個會話記錄")

            return JSONResponse(
                content={
                    "status": "success",
                    "message": f"會話歷史已保存（{session_count} 個會話）",
                }
            )

        except Exception as e:
            debug_log(f"保存會話歷史失敗: {e}")
            return JSONResponse(
                status_code=500,
                content={"status": "error", "message": f"保存失敗: {e!s}"},
            )

    @manager.app.get("/api/active-tabs")
    async def get_active_tabs():
        """獲取活躍標籤頁信息 - 優先使用全局狀態"""
        current_time = time.time()
        expired_threshold = 60

        # 清理過期的全局標籤頁
        valid_global_tabs = {}
        for tab_id, tab_info in manager.global_active_tabs.items():
            if current_time - tab_info.get("last_seen", 0) <= expired_threshold:
                valid_global_tabs[tab_id] = tab_info

        manager.global_active_tabs = valid_global_tabs

        # 如果有當前會話，也更新會話的標籤頁狀態
        current_session = manager.get_current_session()
        if current_session:
            # 合併會話標籤頁到全局（如果有的話）
            session_tabs = getattr(current_session, "active_tabs", {})
            for tab_id, tab_info in session_tabs.items():
                if current_time - tab_info.get("last_seen", 0) <= expired_threshold:
                    valid_global_tabs[tab_id] = tab_info

            # 更新會話的活躍標籤頁
            current_session.active_tabs = valid_global_tabs.copy()
            manager.global_active_tabs = valid_global_tabs

        return JSONResponse(
            content={
                "has_session": current_session is not None,
                "active_tabs": valid_global_tabs,
                "count": len(valid_global_tabs),
            }
        )

    @manager.app.post("/api/register-tab")
    async def register_tab(request: Request):
        """註冊新標籤頁"""
        try:
            data = await request.json()
            tab_id = data.get("tabId")

            if not tab_id:
                return JSONResponse(status_code=400, content={"error": "缺少 tabId"})

            current_session = manager.get_current_session()
            if not current_session:
                return JSONResponse(status_code=404, content={"error": "沒有活躍會話"})

            # 註冊標籤頁
            tab_info = {
                "timestamp": time.time() * 1000,  # 毫秒時間戳
                "last_seen": time.time(),
                "registered_at": time.time(),
            }

            if not hasattr(current_session, "active_tabs"):
                current_session.active_tabs = {}

            current_session.active_tabs[tab_id] = tab_info

            # 同時更新全局標籤頁狀態
            manager.global_active_tabs[tab_id] = tab_info

            debug_log(f"標籤頁已註冊: {tab_id}")

            return JSONResponse(
                content={"status": "success", "tabId": tab_id, "registered": True}
            )

        except Exception as e:
            debug_log(f"註冊標籤頁失敗: {e}")
            return JSONResponse(status_code=500, content={"error": f"註冊失敗: {e!s}"})


async def handle_websocket_message(manager: "WebUIManager", session, data: dict):
    """處理 WebSocket 消息"""
    message_type = data.get("type")

    if message_type == "submit_feedback":
        # 提交回饋
        feedback = data.get("feedback", "")
        images = data.get("images", [])
        settings = data.get("settings", {})
        await session.submit_feedback(feedback, images, settings)

    elif message_type == "run_command":
        # 執行命令
        command = data.get("command", "")
        if command.strip():
            await session.run_command(command)

    elif message_type == "get_status":
        # 獲取會話狀態
        if session.websocket:
            try:
                await session.websocket.send_json(
                    {"type": "status_update", "status_info": session.get_status_info()}
                )
            except Exception as e:
                debug_log(f"發送狀態更新失敗: {e}")

    elif message_type == "heartbeat":
        # WebSocket 心跳處理
        tab_id = data.get("tabId", "unknown")
        timestamp = data.get("timestamp", 0)

        tab_info = {"timestamp": timestamp, "last_seen": time.time()}

        # 更新會話的標籤頁信息
        if hasattr(session, "active_tabs"):
            session.active_tabs[tab_id] = tab_info
        else:
            session.active_tabs = {tab_id: tab_info}

        # 同時更新全局標籤頁狀態
        manager.global_active_tabs[tab_id] = tab_info

        # 發送心跳回應
        if session.websocket:
            try:
                await session.websocket.send_json(
                    {
                        "type": "heartbeat_response",
                        "tabId": tab_id,
                        "timestamp": timestamp,
                    }
                )
            except Exception as e:
                debug_log(f"發送心跳回應失敗: {e}")

    elif message_type == "user_timeout":
        # 用戶設置的超時已到
        debug_log(f"收到用戶超時通知: {session.session_id}")
        # 清理會話資源
        await session._cleanup_resources_on_timeout()
        # 重構：不再自動停止服務器，保持服務器運行以支援持久性

    else:
        debug_log(f"未知的消息類型: {message_type}")


async def _delayed_server_stop(manager: "WebUIManager"):
    """延遲停止服務器"""
    import asyncio

    await asyncio.sleep(5)  # 等待 5 秒讓前端有時間關閉
    from ..main import stop_web_ui

    stop_web_ui()
    debug_log("Web UI 服務器已因用戶超時而停止")



#!/usr/bin/env python3
"""
MCP Feedback Enhanced Desktop Application
=========================================

基於 Tauri 的桌面應用程式包裝器，為 MCP Feedback Enhanced 提供原生桌面體驗。

主要功能：
- 原生桌面應用程式界面
- 整合現有的 Web UI 功能
- 跨平台支援（Windows、macOS、Linux）
- 無需瀏覽器的獨立運行環境

作者: Minidoracat
版本: 2.4.3
"""

__version__ = "2.4.3"
__author__ = "Minidoracat"
__email__ = "minidora0702@gmail.com"

from .desktop_app import DesktopApp, launch_desktop_app


__all__ = [
    "DesktopApp",
    "__author__",
    "__version__",
    "launch_desktop_app",
]



#!/usr/bin/env python3
"""
Web UI 工具模組
==============

提供 Web UI 相關的工具函數。
"""

from .browser import get_browser_opener
from .network import find_free_port


__all__ = ["find_free_port", "get_browser_opener"]



"""
端口管理工具模組

提供增強的端口管理功能，包括：
- 智能端口查找
- 進程檢測和清理
- 端口衝突解決
"""

import socket
import time
from typing import Any

import psutil

from ...debug import debug_log


class PortManager:
    """端口管理器 - 提供增強的端口管理功能"""

    @staticmethod
    def find_process_using_port(port: int) -> dict[str, Any] | None:
        """
        查找占用指定端口的進程

        Args:
            port: 要檢查的端口號

        Returns:
            Dict[str, Any]: 進程信息字典，包含 pid, name, cmdline 等
            None: 如果沒有進程占用該端口
        """
        try:
            for conn in psutil.net_connections(kind="inet"):
                if conn.laddr.port == port and conn.status == psutil.CONN_LISTEN:
                    try:
                        process = psutil.Process(conn.pid)
                        return {
                            "pid": conn.pid,
                            "name": process.name(),
                            "cmdline": " ".join(process.cmdline()),
                            "create_time": process.create_time(),
                            "status": process.status(),
                        }
                    except (psutil.NoSuchProcess, psutil.AccessDenied):
                        # 進程可能已經結束或無權限訪問
                        continue
        except Exception as e:
            debug_log(f"查找端口 {port} 占用進程時發生錯誤: {e}")

        return None

    @staticmethod
    def kill_process_on_port(port: int, force: bool = False) -> bool:
        """
        終止占用指定端口的進程

        Args:
            port: 要清理的端口號
            force: 是否強制終止進程

        Returns:
            bool: 是否成功終止進程
        """
        process_info = PortManager.find_process_using_port(port)
        if not process_info:
            debug_log(f"端口 {port} 沒有被任何進程占用")
            return True

        try:
            pid = process_info["pid"]
            process = psutil.Process(pid)
            process_name = process_info["name"]

            debug_log(f"發現進程 {process_name} (PID: {pid}) 占用端口 {port}")

            # 檢查是否是自己的進程（避免誤殺）
            if "mcp-feedback-enhanced" in process_info["cmdline"].lower():
                debug_log("檢測到 MCP Feedback Enhanced 相關進程，嘗試優雅終止")

            if force:
                debug_log(f"強制終止進程 {process_name} (PID: {pid})")
                process.kill()
            else:
                debug_log(f"優雅終止進程 {process_name} (PID: {pid})")
                process.terminate()

            # 等待進程結束
            try:
                process.wait(timeout=5)
                debug_log(f"成功終止進程 {process_name} (PID: {pid})")
                return True
            except psutil.TimeoutExpired:
                if not force:
                    debug_log(f"優雅終止超時，強制終止進程 {process_name} (PID: {pid})")
                    process.kill()
                    process.wait(timeout=3)
                    return True
                debug_log(f"強制終止進程 {process_name} (PID: {pid}) 失敗")
                return False

        except (psutil.NoSuchProcess, psutil.AccessDenied) as e:
            debug_log(f"無法終止進程 (PID: {process_info['pid']}): {e}")
            return False
        except Exception as e:
            debug_log(f"終止端口 {port} 占用進程時發生錯誤: {e}")
            return False

    @staticmethod
    def is_port_available(host: str, port: int) -> bool:
        """
        檢查端口是否可用

        Args:
            host: 主機地址
            port: 端口號

        Returns:
            bool: 端口是否可用
        """
        try:
            # 首先嘗試不使用 SO_REUSEADDR 來檢測端口
            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
                sock.bind((host, port))
                return True
        except OSError:
            # 如果綁定失敗，再檢查是否真的有進程在監聽
            # 使用 psutil 檢查是否有進程在監聽該端口
            try:
                import psutil

                for conn in psutil.net_connections(kind="inet"):
                    if (
                        conn.laddr.port == port
                        and conn.laddr.ip in [host, "0.0.0.0", "::"]
                        and conn.status == psutil.CONN_LISTEN
                    ):
                        return False
                # 沒有找到監聽的進程，可能是臨時占用，認為可用
                return True
            except Exception:
                # 如果 psutil 檢查失敗，保守地認為端口不可用
                return False

    @staticmethod
    def find_free_port_enhanced(
        preferred_port: int = 8765,
        auto_cleanup: bool = True,
        host: str = "127.0.0.1",
        max_attempts: int = 100,
    ) -> int:
        """
        增強的端口查找功能

        Args:
            preferred_port: 偏好端口號
            auto_cleanup: 是否自動清理占用端口的進程
            host: 主機地址
            max_attempts: 最大嘗試次數

        Returns:
            int: 可用的端口號

        Raises:
            RuntimeError: 如果找不到可用端口
        """
        # 首先嘗試偏好端口
        if PortManager.is_port_available(host, preferred_port):
            debug_log(f"偏好端口 {preferred_port} 可用")
            return preferred_port

        # 如果偏好端口被占用且啟用自動清理
        if auto_cleanup:
            debug_log(f"偏好端口 {preferred_port} 被占用，嘗試清理占用進程")
            process_info = PortManager.find_process_using_port(preferred_port)

            if process_info:
                debug_log(
                    f"端口 {preferred_port} 被進程 {process_info['name']} (PID: {process_info['pid']}) 占用"
                )

                # 詢問用戶是否清理（在實際使用中可能需要配置選項）
                if PortManager._should_cleanup_process(process_info):
                    if PortManager.kill_process_on_port(preferred_port):
                        # 等待一下讓端口釋放
                        time.sleep(1)
                        if PortManager.is_port_available(host, preferred_port):
                            debug_log(f"成功清理端口 {preferred_port}，現在可用")
                            return preferred_port

        # 如果偏好端口仍不可用，尋找其他端口
        debug_log(f"偏好端口 {preferred_port} 不可用，尋找其他可用端口")

        for i in range(max_attempts):
            port = preferred_port + i + 1
            if PortManager.is_port_available(host, port):
                debug_log(f"找到可用端口: {port}")
                return port

        # 如果向上查找失敗，嘗試向下查找
        for i in range(1, min(preferred_port - 1024, max_attempts)):
            port = preferred_port - i
            if port < 1024:  # 避免使用系統保留端口
                break
            if PortManager.is_port_available(host, port):
                debug_log(f"找到可用端口: {port}")
                return port

        raise RuntimeError(
            f"無法在 {preferred_port}±{max_attempts} 範圍內找到可用端口。"
            f"請檢查是否有過多進程占用端口，或手動指定其他端口。"
        )

    @staticmethod
    def _should_cleanup_process(process_info: dict[str, Any]) -> bool:
        """
        判斷是否應該清理指定進程

        Args:
            process_info: 進程信息字典

        Returns:
            bool: 是否應該清理該進程
        """
        # 檢查是否是 MCP Feedback Enhanced 相關進程
        cmdline = process_info.get("cmdline", "").lower()
        process_name = process_info.get("name", "").lower()

        # 如果是自己的進程，允許清理
        if any(
            keyword in cmdline
            for keyword in ["mcp-feedback-enhanced", "mcp_feedback_enhanced"]
        ):
            return True

        # 如果是 Python 進程且命令行包含相關關鍵字
        if "python" in process_name and any(
            keyword in cmdline for keyword in ["uvicorn", "fastapi"]
        ):
            return True

        # 其他情況下，為了安全起見，不自動清理
        debug_log(
            f"進程 {process_info['name']} (PID: {process_info['pid']}) 不是 MCP 相關進程，跳過自動清理"
        )
        return False

    @staticmethod
    def get_port_status(port: int, host: str = "127.0.0.1") -> dict[str, Any]:
        """
        獲取端口狀態信息

        Args:
            port: 端口號
            host: 主機地址

        Returns:
            Dict[str, Any]: 端口狀態信息
        """
        status = {
            "port": port,
            "host": host,
            "available": False,
            "process": None,
            "error": None,
        }

        try:
            # 檢查端口是否可用
            status["available"] = PortManager.is_port_available(host, port)

            # 如果不可用，查找占用進程
            if not status["available"]:
                status["process"] = PortManager.find_process_using_port(port)

        except Exception as e:
            status["error"] = str(e)
            debug_log(f"獲取端口 {port} 狀態時發生錯誤: {e}")

        return status

    @staticmethod
    def list_listening_ports(
        start_port: int = 8000, end_port: int = 9000
    ) -> list[dict[str, Any]]:
        """
        列出指定範圍內正在監聽的端口

        Args:
            start_port: 起始端口
            end_port: 結束端口

        Returns:
            List[Dict[str, Any]]: 監聽端口列表
        """
        listening_ports = []

        try:
            for conn in psutil.net_connections(kind="inet"):
                if (
                    conn.status == psutil.CONN_LISTEN
                    and start_port <= conn.laddr.port <= end_port
                ):
                    try:
                        process = psutil.Process(conn.pid)
                        port_info = {
                            "port": conn.laddr.port,
                            "host": conn.laddr.ip,
                            "pid": conn.pid,
                            "process_name": process.name(),
                            "cmdline": " ".join(process.cmdline()),
                        }
                        listening_ports.append(port_info)
                    except (psutil.NoSuchProcess, psutil.AccessDenied):
                        continue

        except Exception as e:
            debug_log(f"列出監聽端口時發生錯誤: {e}")

        return listening_ports



#!/usr/bin/env python3
"""
集成式內存監控系統
==================

提供與資源管理器深度集成的內存監控功能，包括：
- 系統和進程內存使用監控
- 智能清理觸發機制
- 內存洩漏檢測和趨勢分析
- 性能優化建議
"""

import gc
import threading
from collections import deque
from collections.abc import Callable
from dataclasses import dataclass
from datetime import datetime
from typing import Any

import psutil

from ..debug import debug_log
from .error_handler import ErrorHandler, ErrorType


@dataclass
class MemorySnapshot:
    """內存快照數據類"""

    timestamp: datetime
    system_total: int  # 系統總內存 (bytes)
    system_available: int  # 系統可用內存 (bytes)
    system_used: int  # 系統已用內存 (bytes)
    system_percent: float  # 系統內存使用率 (%)
    process_rss: int  # 進程常駐內存 (bytes)
    process_vms: int  # 進程虛擬內存 (bytes)
    process_percent: float  # 進程內存使用率 (%)
    gc_objects: int  # Python 垃圾回收對象數量


@dataclass
class MemoryAlert:
    """內存警告數據類"""

    level: str  # warning, critical, emergency
    message: str
    timestamp: datetime
    memory_percent: float
    recommended_action: str


@dataclass
class MemoryStats:
    """內存統計數據類"""

    monitoring_duration: float  # 監控持續時間 (秒)
    snapshots_count: int  # 快照數量
    average_system_usage: float  # 平均系統內存使用率
    peak_system_usage: float  # 峰值系統內存使用率
    average_process_usage: float  # 平均進程內存使用率
    peak_process_usage: float  # 峰值進程內存使用率
    alerts_count: int  # 警告數量
    cleanup_triggers: int  # 清理觸發次數
    memory_trend: str  # 內存趨勢 (stable, increasing, decreasing)


class MemoryMonitor:
    """集成式內存監控器"""

    def __init__(
        self,
        warning_threshold: float = 0.8,
        critical_threshold: float = 0.9,
        emergency_threshold: float = 0.95,
        monitoring_interval: int = 30,
        max_snapshots: int = 1000,
    ):
        """
        初始化內存監控器

        Args:
            warning_threshold: 警告閾值 (0.0-1.0)
            critical_threshold: 危險閾值 (0.0-1.0)
            emergency_threshold: 緊急閾值 (0.0-1.0)
            monitoring_interval: 監控間隔 (秒)
            max_snapshots: 最大快照數量
        """
        self.warning_threshold = warning_threshold
        self.critical_threshold = critical_threshold
        self.emergency_threshold = emergency_threshold
        self.monitoring_interval = monitoring_interval
        self.max_snapshots = max_snapshots

        # 監控狀態
        self.is_monitoring = False
        self.monitor_thread: threading.Thread | None = None
        self._stop_event = threading.Event()

        # 數據存儲
        self.snapshots: deque = deque(maxlen=max_snapshots)
        self.alerts: list[MemoryAlert] = []
        self.max_alerts = 100

        # 回調函數
        self.cleanup_callbacks: list[Callable] = []
        self.alert_callbacks: list[Callable[[MemoryAlert], None]] = []

        # 統計數據
        self.start_time: datetime | None = None
        self.cleanup_triggers_count = 0

        # 進程信息
        self.process = psutil.Process()

        debug_log("MemoryMonitor 初始化完成")

    def start_monitoring(self) -> bool:
        """
        開始內存監控

        Returns:
            bool: 是否成功啟動
        """
        if self.is_monitoring:
            debug_log("內存監控已在運行")
            return True

        try:
            self.is_monitoring = True
            self.start_time = datetime.now()
            self._stop_event.clear()

            self.monitor_thread = threading.Thread(
                target=self._monitoring_loop, name="MemoryMonitor", daemon=True
            )
            self.monitor_thread.start()

            debug_log(f"內存監控已啟動，間隔 {self.monitoring_interval} 秒")
            return True

        except Exception as e:
            self.is_monitoring = False
            error_id = ErrorHandler.log_error_with_context(
                e, context={"operation": "啟動內存監控"}, error_type=ErrorType.SYSTEM
            )
            debug_log(f"啟動內存監控失敗 [錯誤ID: {error_id}]: {e}")
            return False

    def stop_monitoring(self) -> bool:
        """
        停止內存監控

        Returns:
            bool: 是否成功停止
        """
        if not self.is_monitoring:
            debug_log("內存監控未在運行")
            return True

        try:
            self.is_monitoring = False
            self._stop_event.set()

            if self.monitor_thread and self.monitor_thread.is_alive():
                self.monitor_thread.join(timeout=5)

            debug_log("內存監控已停止")
            return True

        except Exception as e:
            error_id = ErrorHandler.log_error_with_context(
                e, context={"operation": "停止內存監控"}, error_type=ErrorType.SYSTEM
            )
            debug_log(f"停止內存監控失敗 [錯誤ID: {error_id}]: {e}")
            return False

    def _monitoring_loop(self):
        """內存監控主循環"""
        debug_log("內存監控循環開始")

        while not self._stop_event.is_set():
            try:
                # 收集內存快照
                snapshot = self._collect_memory_snapshot()
                self.snapshots.append(snapshot)

                # 檢查內存使用情況
                self._check_memory_usage(snapshot)

                # 等待下次監控
                if self._stop_event.wait(self.monitoring_interval):
                    break

            except Exception as e:
                error_id = ErrorHandler.log_error_with_context(
                    e,
                    context={"operation": "內存監控循環"},
                    error_type=ErrorType.SYSTEM,
                )
                debug_log(f"內存監控循環錯誤 [錯誤ID: {error_id}]: {e}")

                # 發生錯誤時等待較短時間後重試
                if self._stop_event.wait(5):
                    break

        debug_log("內存監控循環結束")

    def _collect_memory_snapshot(self) -> MemorySnapshot:
        """收集內存快照"""
        try:
            # 系統內存信息
            system_memory = psutil.virtual_memory()

            # 進程內存信息
            process_memory = self.process.memory_info()
            process_percent = self.process.memory_percent()

            # Python 垃圾回收信息
            gc_objects = len(gc.get_objects())

            return MemorySnapshot(
                timestamp=datetime.now(),
                system_total=system_memory.total,
                system_available=system_memory.available,
                system_used=system_memory.used,
                system_percent=system_memory.percent,
                process_rss=process_memory.rss,
                process_vms=process_memory.vms,
                process_percent=process_percent,
                gc_objects=gc_objects,
            )

        except Exception as e:
            error_id = ErrorHandler.log_error_with_context(
                e, context={"operation": "收集內存快照"}, error_type=ErrorType.SYSTEM
            )
            debug_log(f"收集內存快照失敗 [錯誤ID: {error_id}]: {e}")
            raise

    def _check_memory_usage(self, snapshot: MemorySnapshot):
        """檢查內存使用情況並觸發相應動作"""
        usage_percent = snapshot.system_percent / 100.0

        # 檢查緊急閾值
        if usage_percent >= self.emergency_threshold:
            alert = MemoryAlert(
                level="emergency",
                message=f"內存使用率達到緊急水平: {snapshot.system_percent:.1f}%",
                timestamp=snapshot.timestamp,
                memory_percent=snapshot.system_percent,
                recommended_action="立即執行強制清理和垃圾回收",
            )
            self._handle_alert(alert)
            self._trigger_emergency_cleanup()

        # 檢查危險閾值
        elif usage_percent >= self.critical_threshold:
            alert = MemoryAlert(
                level="critical",
                message=f"內存使用率達到危險水平: {snapshot.system_percent:.1f}%",
                timestamp=snapshot.timestamp,
                memory_percent=snapshot.system_percent,
                recommended_action="執行資源清理和垃圾回收",
            )
            self._handle_alert(alert)
            self._trigger_cleanup()

        # 檢查警告閾值
        elif usage_percent >= self.warning_threshold:
            alert = MemoryAlert(
                level="warning",
                message=f"內存使用率較高: {snapshot.system_percent:.1f}%",
                timestamp=snapshot.timestamp,
                memory_percent=snapshot.system_percent,
                recommended_action="考慮執行輕量級清理",
            )
            self._handle_alert(alert)

    def _handle_alert(self, alert: MemoryAlert):
        """處理內存警告"""
        # 添加到警告列表
        self.alerts.append(alert)

        # 限制警告數量
        if len(self.alerts) > self.max_alerts:
            self.alerts = self.alerts[-self.max_alerts :]

        # 調用警告回調
        for callback in self.alert_callbacks:
            try:
                callback(alert)
            except Exception as e:
                debug_log(f"警告回調執行失敗: {e}")

        debug_log(f"內存警告 [{alert.level}]: {alert.message}")

    def _trigger_cleanup(self):
        """觸發清理操作"""
        self.cleanup_triggers_count += 1
        debug_log("觸發內存清理操作")

        # 執行 Python 垃圾回收
        collected = gc.collect()
        debug_log(f"垃圾回收清理了 {collected} 個對象")

        # 調用清理回調
        for callback in self.cleanup_callbacks:
            try:
                callback()
            except Exception as e:
                debug_log(f"清理回調執行失敗: {e}")

    def _trigger_emergency_cleanup(self):
        """觸發緊急清理操作"""
        debug_log("觸發緊急內存清理操作")

        # 執行強制垃圾回收
        for _ in range(3):
            collected = gc.collect()
            debug_log(f"強制垃圾回收清理了 {collected} 個對象")

        # 調用清理回調（強制模式）
        for callback in self.cleanup_callbacks:
            try:
                # 修復 unreachable 錯誤 - 簡化邏輯，移除不可達的 else 分支
                # 嘗試傳遞 force 參數
                import inspect

                sig = inspect.signature(callback)
                if "force" in sig.parameters:
                    callback(force=True)
                else:
                    callback()
            except Exception as e:
                debug_log(f"緊急清理回調執行失敗: {e}")

    def add_cleanup_callback(self, callback: Callable):
        """添加清理回調函數"""
        if callback not in self.cleanup_callbacks:
            self.cleanup_callbacks.append(callback)
            debug_log("添加清理回調函數")

    def add_alert_callback(self, callback: Callable[[MemoryAlert], None]):
        """添加警告回調函數"""
        if callback not in self.alert_callbacks:
            self.alert_callbacks.append(callback)
            debug_log("添加警告回調函數")

    def remove_cleanup_callback(self, callback: Callable):
        """移除清理回調函數"""
        if callback in self.cleanup_callbacks:
            self.cleanup_callbacks.remove(callback)
            debug_log("移除清理回調函數")

    def remove_alert_callback(self, callback: Callable[[MemoryAlert], None]):
        """移除警告回調函數"""
        if callback in self.alert_callbacks:
            self.alert_callbacks.remove(callback)
            debug_log("移除警告回調函數")

    def get_current_memory_info(self) -> dict[str, Any]:
        """獲取當前內存信息"""
        try:
            snapshot = self._collect_memory_snapshot()
            return {
                "timestamp": snapshot.timestamp.isoformat(),
                "system": {
                    "total_gb": round(snapshot.system_total / (1024**3), 2),
                    "available_gb": round(snapshot.system_available / (1024**3), 2),
                    "used_gb": round(snapshot.system_used / (1024**3), 2),
                    "usage_percent": round(snapshot.system_percent, 1),
                },
                "process": {
                    "rss_mb": round(snapshot.process_rss / (1024**2), 2),
                    "vms_mb": round(snapshot.process_vms / (1024**2), 2),
                    "usage_percent": round(snapshot.process_percent, 1),
                },
                "gc_objects": snapshot.gc_objects,
                "status": self._get_memory_status(snapshot.system_percent / 100.0),
            }
        except Exception as e:
            error_id = ErrorHandler.log_error_with_context(
                e,
                context={"operation": "獲取當前內存信息"},
                error_type=ErrorType.SYSTEM,
            )
            debug_log(f"獲取內存信息失敗 [錯誤ID: {error_id}]: {e}")
            return {}

    def get_memory_stats(self) -> MemoryStats:
        """獲取內存統計數據"""
        if not self.snapshots:
            return MemoryStats(
                monitoring_duration=0.0,
                snapshots_count=0,
                average_system_usage=0.0,
                peak_system_usage=0.0,
                average_process_usage=0.0,
                peak_process_usage=0.0,
                alerts_count=0,
                cleanup_triggers=0,
                memory_trend="unknown",
            )

        # 計算統計數據
        system_usages = [s.system_percent for s in self.snapshots]
        process_usages = [s.process_percent for s in self.snapshots]

        duration = 0.0
        if self.start_time:
            duration = (datetime.now() - self.start_time).total_seconds()

        return MemoryStats(
            monitoring_duration=duration,
            snapshots_count=len(self.snapshots),
            average_system_usage=sum(system_usages) / len(system_usages),
            peak_system_usage=max(system_usages),
            average_process_usage=sum(process_usages) / len(process_usages),
            peak_process_usage=max(process_usages),
            alerts_count=len(self.alerts),
            cleanup_triggers=self.cleanup_triggers_count,
            memory_trend=self._analyze_memory_trend(),
        )

    def get_recent_alerts(self, limit: int = 10) -> list[MemoryAlert]:
        """獲取最近的警告"""
        return self.alerts[-limit:] if self.alerts else []

    def _get_memory_status(self, usage_percent: float) -> str:
        """獲取內存狀態描述"""
        if usage_percent >= self.emergency_threshold:
            return "emergency"
        if usage_percent >= self.critical_threshold:
            return "critical"
        if usage_percent >= self.warning_threshold:
            return "warning"
        return "normal"

    def _analyze_memory_trend(self) -> str:
        """分析內存使用趨勢"""
        if len(self.snapshots) < 10:
            return "insufficient_data"

        # 取最近的快照進行趨勢分析
        recent_snapshots = list(self.snapshots)[-10:]
        usages = [s.system_percent for s in recent_snapshots]

        # 簡單的線性趨勢分析
        first_half = usages[:5]
        second_half = usages[5:]

        avg_first = sum(first_half) / len(first_half)
        avg_second = sum(second_half) / len(second_half)

        diff = avg_second - avg_first

        if abs(diff) < 2.0:  # 變化小於 2%
            return "stable"
        if diff > 0:
            return "increasing"
        return "decreasing"

    def force_cleanup(self):
        """手動觸發清理操作"""
        debug_log("手動觸發內存清理")
        self._trigger_cleanup()

    def force_emergency_cleanup(self):
        """手動觸發緊急清理操作"""
        debug_log("手動觸發緊急內存清理")
        self._trigger_emergency_cleanup()

    def reset_stats(self):
        """重置統計數據"""
        self.snapshots.clear()
        self.alerts.clear()
        self.cleanup_triggers_count = 0
        self.start_time = datetime.now() if self.is_monitoring else None
        debug_log("內存監控統計數據已重置")

    def export_memory_data(self) -> dict[str, Any]:
        """導出內存數據"""
        return {
            "config": {
                "warning_threshold": self.warning_threshold,
                "critical_threshold": self.critical_threshold,
                "emergency_threshold": self.emergency_threshold,
                "monitoring_interval": self.monitoring_interval,
            },
            "current_info": self.get_current_memory_info(),
            "stats": self.get_memory_stats().__dict__,
            "recent_alerts": [
                {
                    "level": alert.level,
                    "message": alert.message,
                    "timestamp": alert.timestamp.isoformat(),
                    "memory_percent": alert.memory_percent,
                    "recommended_action": alert.recommended_action,
                }
                for alert in self.get_recent_alerts()
            ],
            "is_monitoring": self.is_monitoring,
        }


# 全域內存監控器實例
_memory_monitor: MemoryMonitor | None = None
_monitor_lock = threading.Lock()


def get_memory_monitor() -> MemoryMonitor:
    """獲取全域內存監控器實例"""
    global _memory_monitor
    if _memory_monitor is None:
        with _monitor_lock:
            if _memory_monitor is None:
                _memory_monitor = MemoryMonitor()
    return _memory_monitor



#!/usr/bin/env python3
"""
Web UI 資料模型模組
==================

定義 Web UI 相關的資料結構和型別。
"""

from .feedback_result import FeedbackResult
from .feedback_session import CleanupReason, SessionStatus, WebFeedbackSession


__all__ = ["CleanupReason", "FeedbackResult", "SessionStatus", "WebFeedbackSession"]



#!/usr/bin/env python3
"""
桌面應用程式構建腳本

此腳本負責構建 Tauri 桌面應用程式和 Python 擴展模組，
確保在 PyPI 發布時包含預編譯的二進制檔案。

使用方法：
    python scripts/build_desktop.py [--release] [--clean]
"""

import argparse
import os
import shutil
import subprocess
import sys
from pathlib import Path


def run_command(
    cmd: list[str], cwd: str = None, check: bool = True, show_info: bool = True
) -> subprocess.CompletedProcess:
    """執行命令並返回結果"""
    if show_info:
        print(f"🔧 執行命令: {' '.join(cmd)}")
        if cwd:
            print(f"📁 工作目錄: {cwd}")

    result = subprocess.run(cmd, cwd=cwd, capture_output=True, text=True, check=False)

    # 處理標準輸出
    if result.stdout and show_info:
        print("📤 輸出:")
        print(result.stdout.strip())

    # 智能處理標準錯誤 - 區分信息和真正的錯誤
    if result.stderr:
        stderr_lines = result.stderr.strip().split("\n")
        info_lines = []
        error_lines = []

        for line in stderr_lines:
            stripped_line = line.strip()
            if not stripped_line:
                continue
            # 識別信息性消息和正常編譯輸出
            if (
                stripped_line.startswith("info:")
                or "is up to date" in stripped_line
                or "downloading component" in stripped_line
                or "installing component" in stripped_line
                or stripped_line.startswith("Compiling")
                or stripped_line.startswith("Finished")
                or stripped_line.startswith("Building")
                or "target(s) in" in stripped_line
            ):
                info_lines.append(stripped_line)
            else:
                error_lines.append(stripped_line)

        # 顯示信息性消息
        if info_lines and show_info:
            print("ℹ️  信息:")
            for line in info_lines:
                print(f"   {line}")

        # 顯示真正的錯誤
        if error_lines:
            print("❌ 錯誤:")
            for line in error_lines:
                print(f"   {line}")

    if check and result.returncode != 0:
        raise subprocess.CalledProcessError(result.returncode, cmd)

    return result


def check_rust_environment():
    """檢查 Rust 開發環境"""
    print("🔍 檢查 Rust 開發環境...")

    try:
        result = run_command(["rustc", "--version"])
        print(f"✅ Rust 編譯器: {result.stdout.strip()}")
    except (subprocess.CalledProcessError, FileNotFoundError):
        print("❌ 未找到 Rust 編譯器")
        print("💡 請安裝 Rust: https://rustup.rs/")
        return False

    try:
        result = run_command(["cargo", "--version"])
        print(f"✅ Cargo: {result.stdout.strip()}")
    except (subprocess.CalledProcessError, FileNotFoundError):
        print("❌ 未找到 Cargo")
        return False

    try:
        result = run_command(["cargo", "install", "--list"])
        if "tauri-cli" in result.stdout:
            print("✅ Tauri CLI 已安裝")
        else:
            print("⚠️  Tauri CLI 未安裝，嘗試安裝...")
            run_command(["cargo", "install", "tauri-cli"])
            print("✅ Tauri CLI 安裝完成")
    except subprocess.CalledProcessError:
        print("❌ 無法安裝 Tauri CLI")
        return False

    return True


def install_rust_targets():
    """安裝跨平台編譯所需的 Rust targets"""
    print("🎯 安裝跨平台編譯 targets...")

    # 定義需要的 targets
    targets = [
        ("x86_64-pc-windows-msvc", "Windows x64"),
        ("x86_64-apple-darwin", "macOS Intel"),
        ("aarch64-apple-darwin", "macOS Apple Silicon"),
        ("x86_64-unknown-linux-gnu", "Linux x64"),
    ]

    installed_count = 0
    updated_count = 0

    for target, description in targets:
        print(f"📦 檢查 target: {target} ({description})")
        try:
            result = run_command(
                ["rustup", "target", "add", target], check=False, show_info=False
            )

            if result.returncode == 0:
                # 檢查是否是新安裝還是已存在
                if "is up to date" in result.stderr:
                    print(f"✅ {description} - 已是最新版本")
                    updated_count += 1
                elif "installing component" in result.stderr:
                    print(f"🆕 {description} - 新安裝完成")
                    installed_count += 1
                else:
                    print(f"✅ {description} - 安裝成功")
                    installed_count += 1
            else:
                print(f"⚠️  {description} - 安裝失敗")
                if result.stderr:
                    print(f"   錯誤: {result.stderr.strip()}")
        except Exception as e:
            print(f"⚠️  安裝 {description} 時發生錯誤: {e}")

    print(
        f"✅ Rust targets 檢查完成 (新安裝: {installed_count}, 已存在: {updated_count})"
    )


def clean_build_artifacts(project_root: Path):
    """清理構建產物"""
    print("🧹 清理構建產物...")

    # 清理 Rust 構建產物
    rust_target = project_root / "src-tauri" / "target"
    if rust_target.exists():
        print(f"清理 Rust target 目錄: {rust_target}")
        shutil.rmtree(rust_target)

    # 清理 Python 構建產物
    python_build_dirs = [
        project_root / "build",
        project_root / "dist",
        project_root / "*.egg-info",
    ]

    for build_dir in python_build_dirs:
        if build_dir.exists():
            print(f"清理 Python 構建目錄: {build_dir}")
            if build_dir.is_dir():
                shutil.rmtree(build_dir)
            else:
                build_dir.unlink()


def build_rust_extension(project_root: Path, release: bool = True):
    """構建 Rust 擴展模組"""
    print("🔨 構建 Rust 擴展模組...")

    src_tauri = project_root / "src-tauri"
    if not src_tauri.exists():
        raise FileNotFoundError(f"src-tauri 目錄不存在: {src_tauri}")

    # 構建 Rust 庫
    build_cmd = ["cargo", "build"]
    if release:
        build_cmd.append("--release")

    run_command(build_cmd, cwd=str(src_tauri))
    print("✅ Rust 擴展模組構建完成")


def build_tauri_app_multiplatform(project_root: Path, release: bool = True):
    """構建多平台 Tauri 桌面應用程式"""
    print("🖥️ 構建多平台 Tauri 桌面應用程式...")

    src_tauri = project_root / "src-tauri"

    # 定義目標平台
    targets = [
        ("x86_64-pc-windows-msvc", "mcp-feedback-enhanced-desktop.exe"),
        ("x86_64-apple-darwin", "mcp-feedback-enhanced-desktop"),
        ("aarch64-apple-darwin", "mcp-feedback-enhanced-desktop"),
        ("x86_64-unknown-linux-gnu", "mcp-feedback-enhanced-desktop"),
    ]

    successful_builds = []

    # 平台描述映射
    platform_descriptions = {
        "x86_64-pc-windows-msvc": "Windows x64",
        "x86_64-apple-darwin": "macOS Intel",
        "aarch64-apple-darwin": "macOS Apple Silicon",
        "x86_64-unknown-linux-gnu": "Linux x64",
    }

    for target, binary_name in targets:
        description = platform_descriptions.get(target, target)
        print(f"🔨 構建 {description} ({target})...")

        # 構建命令
        build_cmd = [
            "cargo",
            "build",
            "--bin",
            "mcp-feedback-enhanced-desktop",
            "--target",
            target,
        ]
        if release:
            build_cmd.append("--release")

        try:
            run_command(build_cmd, cwd=str(src_tauri), show_info=False)
            successful_builds.append((target, binary_name))
            print(f"✅ {description} 構建成功")
        except subprocess.CalledProcessError as e:
            print(f"⚠️  {description} 構建失敗")
            print("💡 可能缺少該平台的編譯工具鏈或依賴")
            # 顯示具體錯誤信息
            if hasattr(e, "stderr") and e.stderr:
                print(f"   錯誤詳情: {e.stderr.strip()}")
        except Exception as e:
            print(f"❌ {description} 構建時發生未預期錯誤: {e}")

    if successful_builds:
        print(f"✅ 成功構建 {len(successful_builds)} 個平台")

        # 如果只構建了當前平台，給出提示
        if len(successful_builds) == 1:
            print("")
            print("💡 注意：只成功構建了當前平台的二進制文件")
            print("   其他平台的構建失敗通常是因為缺少跨平台編譯工具鏈")
            print("   完整的多平台支援將在 GitHub Actions CI 中完成")
            print("   發佈到 PyPI 時會包含所有平台的二進制文件")

        return successful_builds
    print("❌ 所有平台構建都失敗了")
    return []


def copy_multiplatform_artifacts(
    project_root: Path, successful_builds: list, release: bool = True
):
    """複製多平台構建產物到適當位置"""
    print("📦 複製多平台構建產物...")

    src_tauri = project_root / "src-tauri"
    build_type = "release" if release else "debug"

    # 創建目標目錄
    desktop_dir = project_root / "src" / "mcp_feedback_enhanced" / "desktop_release"
    desktop_dir.mkdir(parents=True, exist_ok=True)

    # 定義平台到文件名的映射
    platform_mapping = {
        "x86_64-pc-windows-msvc": "mcp-feedback-enhanced-desktop.exe",
        "x86_64-apple-darwin": "mcp-feedback-enhanced-desktop-macos-intel",
        "aarch64-apple-darwin": "mcp-feedback-enhanced-desktop-macos-arm64",
        "x86_64-unknown-linux-gnu": "mcp-feedback-enhanced-desktop-linux",
    }

    copied_files = []

    for target, original_binary_name in successful_builds:
        # 源文件路徑
        src_file = src_tauri / "target" / target / build_type / original_binary_name

        # 目標文件名
        dst_filename = platform_mapping.get(target, original_binary_name)
        dst_file = desktop_dir / dst_filename

        if src_file.exists():
            shutil.copy2(src_file, dst_file)
            # 設置執行權限（非 Windows）
            # 0o755 權限是必要的，因為這些是可執行的二進制檔案
            if not dst_filename.endswith(".exe"):
                os.chmod(dst_file, 0o755)  # noqa: S103
            copied_files.append(dst_filename)
            print(f"✅ 複製 {target} 二進制檔案: {src_file} -> {dst_file}")
        else:
            print(f"⚠️  找不到 {target} 的二進制檔案: {src_file}")

    if not copied_files:
        print("⚠️  沒有找到可複製的二進制檔案")
        return False

    # 創建 __init__.py 文件，讓 desktop 目錄成為 Python 包
    desktop_init = desktop_dir / "__init__.py"
    if not desktop_init.exists():
        desktop_init.write_text('"""桌面應用程式二進制檔案"""', encoding="utf-8")
        print(f"✅ 創建 __init__.py: {desktop_init}")

    print(f"✅ 成功複製 {len(copied_files)} 個平台的二進制檔案")
    return True


def copy_desktop_python_module(project_root: Path):
    """複製桌面應用 Python 模組到發佈位置"""
    print("📦 複製桌面應用 Python 模組...")

    # 源路徑和目標路徑
    python_src = project_root / "src-tauri" / "python" / "mcp_feedback_enhanced_desktop"
    python_dst = project_root / "src" / "mcp_feedback_enhanced" / "desktop_app"

    if not python_src.exists():
        print(f"⚠️  源模組不存在: {python_src}")
        return False

    # 如果目標目錄存在，先刪除
    if python_dst.exists():
        shutil.rmtree(python_dst)
        print(f"🗑️  清理舊的模組目錄: {python_dst}")

    # 複製模組
    shutil.copytree(python_src, python_dst)
    print(f"✅ 複製桌面應用模組: {python_src} -> {python_dst}")

    return True


def main():
    """主函數"""
    parser = argparse.ArgumentParser(
        description="構建 MCP Feedback Enhanced 桌面應用程式",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
範例:
  python scripts/build_desktop.py                # 構建 Debug 版本
  python scripts/build_desktop.py --release      # 構建 Release 版本
  python scripts/build_desktop.py --clean        # 清理構建產物

構建完成後，可以使用以下命令測試:
  python -m mcp_feedback_enhanced test --desktop

或使用 Makefile:
  make build-desktop          # 構建 Debug 版本
  make build-desktop-release  # 構建 Release 版本
  make test-desktop          # 構建並測試
        """,
    )
    parser.add_argument(
        "--release", action="store_true", help="構建發布版本 (預設為 Debug)"
    )
    parser.add_argument("--clean", action="store_true", help="清理構建產物")

    args = parser.parse_args()

    # 獲取專案根目錄
    project_root = Path(__file__).parent.parent.resolve()
    print(f"專案根目錄: {project_root}")

    try:
        # 清理構建產物（如果需要）
        if args.clean:
            clean_build_artifacts(project_root)

        # 檢查 Rust 環境
        if not check_rust_environment():
            sys.exit(1)

        # 安裝跨平台編譯 targets
        install_rust_targets()

        # 構建 Rust 擴展
        build_rust_extension(project_root, args.release)

        # 構建多平台 Tauri 應用程式
        successful_builds = build_tauri_app_multiplatform(project_root, args.release)

        if not successful_builds:
            print("❌ 沒有成功構建任何平台")
            sys.exit(1)

        # 複製多平台構建產物
        if not copy_multiplatform_artifacts(
            project_root, successful_builds, args.release
        ):
            print("⚠️  構建產物複製失敗，但 Rust 編譯成功")
            return

        # 複製桌面應用 Python 模組
        if not copy_desktop_python_module(project_root):
            print("⚠️  桌面應用模組複製失敗")
            return

        print("🎉 多平台桌面應用程式構建完成！")
        print("")
        print("📍 構建產物位置:")
        print("   多平台二進制檔案: src/mcp_feedback_enhanced/desktop_release/")
        print("   桌面應用模組: src/mcp_feedback_enhanced/desktop_app/")
        print("   開發環境模組: src-tauri/python/mcp_feedback_enhanced_desktop/")
        print("")
        print("🌍 支援的平台:")
        for target, _ in successful_builds:
            print(f"   ✅ {target}")
        print("")
        print("🚀 下一步:")
        print("   測試桌面應用程式: python -m mcp_feedback_enhanced test --desktop")
        print("   或使用 Makefile: make test-desktop")
        print("   構建發布包: make build-all")

    except Exception as e:
        print(f"❌ 構建失敗: {e}")
        sys.exit(1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
MCP Feedback Enhanced Web UI 模組

基於 FastAPI 和 WebSocket 的 Web 用戶介面，提供豐富的互動回饋功能。
支援文字輸入、圖片上傳、命令執行等功能，設計採用現代化的 Web UI 架構。

主要功能：
- FastAPI Web 應用程式
- WebSocket 實時通訊
- 多語言國際化支援
- 圖片上傳與預覽
- 命令執行與結果展示
- 響應式設計
- 本地和遠端環境適配
"""

from .main import WebUIManager, get_web_ui_manager, launch_web_feedback_ui, stop_web_ui


__all__ = [
    "WebUIManager",
    "get_web_ui_manager",
    "launch_web_feedback_ui",
    "stop_web_ui",
]



#!/usr/bin/env python3
"""
生成基本的應用程式圖標

這個腳本會生成 Tauri 應用程式所需的基本圖標文件。
在實際部署中，應該使用專業的圖標設計。
"""

import os

from PIL import Image, ImageDraw


def create_simple_icon(size, output_path):
    """創建簡單的圖標"""
    # 創建圖像
    img = Image.new("RGBA", (size, size), (0, 0, 0, 0))
    draw = ImageDraw.Draw(img)

    # 繪製簡單的圖標 - 一個帶邊框的圓形
    margin = size // 8
    draw.ellipse(
        [margin, margin, size - margin, size - margin],
        fill=(52, 152, 219, 255),
        outline=(41, 128, 185, 255),
        width=2,
    )

    # 在中心繪製 "MCP" 文字
    try:
        # 嘗試使用系統字體
        from PIL import ImageFont

        font_size = size // 4
        try:
            font = ImageFont.truetype("arial.ttf", font_size)
        except:
            font = ImageFont.load_default()
    except ImportError:
        font = None

    text = "MCP"
    if font:
        bbox = draw.textbbox((0, 0), text, font=font)
        text_width = bbox[2] - bbox[0]
        text_height = bbox[3] - bbox[1]
    else:
        text_width = len(text) * (size // 8)
        text_height = size // 6

    text_x = (size - text_width) // 2
    text_y = (size - text_height) // 2

    draw.text((text_x, text_y), text, fill=(255, 255, 255, 255), font=font)

    # 保存圖像
    img.save(output_path)
    print(f"已生成圖標: {output_path}")


def main():
    """主函數"""
    icons_dir = "icons"
    os.makedirs(icons_dir, exist_ok=True)

    # 生成不同尺寸的 PNG 圖標
    sizes = [32, 128, 256]
    for size in sizes:
        if size == 128:
            # 生成普通和 2x 版本
            create_simple_icon(size, f"{icons_dir}/{size}x{size}.png")
            create_simple_icon(size * 2, f"{icons_dir}/{size}x{size}@2x.png")
        else:
            create_simple_icon(size, f"{icons_dir}/{size}x{size}.png")

    # 為 Windows 創建 ICO 文件
    try:
        img_256 = Image.open(f"{icons_dir}/256x256.png")
        img_256.save(
            f"{icons_dir}/icon.ico",
            format="ICO",
            sizes=[(256, 256), (128, 128), (64, 64), (32, 32), (16, 16)],
        )
        print(f"已生成 Windows 圖標: {icons_dir}/icon.ico")
    except Exception as e:
        print(f"生成 ICO 文件失敗: {e}")

    # 為 macOS 創建 ICNS 文件（需要額外工具）
    print("注意：macOS ICNS 文件需要使用專門的工具生成")
    print("可以使用在線工具或 iconutil 命令將 PNG 轉換為 ICNS")

    print("圖標生成完成！")


if __name__ == "__main__":
    try:
        main()
    except ImportError:
        print("錯誤：需要安裝 Pillow 庫")
        print("請運行：pip install Pillow")
    except Exception as e:
        print(f"生成圖標時發生錯誤: {e}")



#!/usr/bin/env python3
"""
UV Cache 清理腳本
================

定期清理 uv cache 以防止磁碟空間不斷增加
特別針對 Windows 系統「檔案正由另一個程序使用」的問題提供解決方案

使用方式：
  python scripts/cleanup_cache.py --size       # 查看 cache 大小和詳細資訊
  python scripts/cleanup_cache.py --dry-run    # 預覽將要清理的內容（不實際清理）
  python scripts/cleanup_cache.py --clean      # 執行標準清理
  python scripts/cleanup_cache.py --force      # 強制清理（會嘗試關閉相關程序）

功能特色：
  - 智能跳過正在使用中的檔案
  - 提供強制清理模式
  - 詳細的清理統計和進度顯示
  - 支援 Windows/macOS/Linux 跨平台
"""

import argparse
import os
import subprocess
from pathlib import Path


def get_cache_dir():
    """取得 uv cache 目錄"""
    # Windows 預設路徑
    if os.name == "nt":
        return Path.home() / "AppData" / "Local" / "uv"
    # macOS/Linux 預設路徑
    return Path.home() / ".cache" / "uv"


def get_cache_size(cache_dir):
    """計算 cache 目錄大小"""
    if not cache_dir.exists():
        return 0

    total_size = 0
    for dirpath, dirnames, filenames in os.walk(cache_dir):
        for filename in filenames:
            filepath = os.path.join(dirpath, filename)
            try:
                total_size += os.path.getsize(filepath)
            except (OSError, FileNotFoundError):
                pass
    return total_size


def format_size(size_bytes):
    """格式化檔案大小顯示"""
    if size_bytes == 0:
        return "0 B"

    for unit in ["B", "KB", "MB", "GB"]:
        if size_bytes < 1024.0:
            return f"{size_bytes:.1f} {unit}"
        size_bytes /= 1024.0
    return f"{size_bytes:.1f} TB"


def run_uv_command(command, check=True):
    """執行 uv 命令"""
    try:
        result = subprocess.run(
            ["uv"] + command, capture_output=True, text=True, check=check
        )
        return result
    except subprocess.CalledProcessError as e:
        print(f"❌ 命令執行失敗: uv {' '.join(command)}")
        print(f"錯誤: {e.stderr}")
        return None
    except FileNotFoundError:
        print("❌ 找不到 uv 命令，請確認 uv 已正確安裝")
        return None


def show_cache_info():
    """顯示 cache 資訊"""
    print("🔍 UV Cache 資訊")
    print("=" * 50)

    cache_dir = get_cache_dir()
    print(f"Cache 目錄: {cache_dir}")

    if cache_dir.exists():
        cache_size = get_cache_size(cache_dir)
        print(f"Cache 大小: {format_size(cache_size)}")

        # 顯示子目錄大小
        subdirs = []
        for subdir in cache_dir.iterdir():
            if subdir.is_dir():
                subdir_size = get_cache_size(subdir)
                subdirs.append((subdir.name, subdir_size))

        if subdirs:
            print("\n📁 子目錄大小:")
            subdirs.sort(key=lambda x: x[1], reverse=True)
            for name, size in subdirs[:10]:  # 顯示前10大
                print(f"  {name}: {format_size(size)}")
    else:
        print("Cache 目錄不存在")


def clean_cache_selective(cache_dir, dry_run=False):
    """選擇性清理 cache，跳過正在使用的檔案"""
    cleaned_count = 0
    skipped_count = 0
    total_saved = 0

    print(f"🔍 掃描 cache 目錄: {cache_dir}")

    # 遍歷 cache 目錄
    for root, dirs, files in os.walk(cache_dir):
        # 跳過一些可能正在使用的目錄
        if any(skip_dir in root for skip_dir in ["Scripts", "Lib", "pyvenv.cfg"]):
            continue

        for file in files:
            file_path = Path(root) / file
            try:
                if dry_run:
                    file_size = file_path.stat().st_size
                    total_saved += file_size
                    cleaned_count += 1
                    if cleaned_count <= 10:  # 只顯示前10個
                        print(
                            f"  將清理: {file_path.relative_to(cache_dir)} ({format_size(file_size)})"
                        )
                else:
                    file_size = file_path.stat().st_size
                    file_path.unlink()
                    total_saved += file_size
                    cleaned_count += 1
            except (OSError, PermissionError, FileNotFoundError):
                skipped_count += 1
                if not dry_run and skipped_count <= 5:  # 只顯示前5個錯誤
                    print(f"  ⚠️  跳過: {file_path.name} (正在使用中)")

    return cleaned_count, skipped_count, total_saved


def clean_cache(dry_run=False):
    """清理 cache"""
    action = "預覽" if dry_run else "執行"
    print(f"🧹 {action} UV Cache 清理")
    print("=" * 50)

    # 顯示清理前的大小
    cache_dir = get_cache_dir()
    if cache_dir.exists():
        before_size = get_cache_size(cache_dir)
        print(f"清理前大小: {format_size(before_size)}")
    else:
        print("Cache 目錄不存在，無需清理")
        return

    if dry_run:
        print("\n🔍 將要清理的內容:")
        # 先嘗試 uv cache clean --dry-run
        result = run_uv_command(["cache", "clean", "--dry-run"], check=False)
        if result and result.returncode == 0:
            print(result.stdout)
        else:
            print("  使用自定義掃描...")
            cleaned_count, skipped_count, total_saved = clean_cache_selective(
                cache_dir, dry_run=True
            )
            print("\n📊 預覽結果:")
            print(f"  可清理檔案: {cleaned_count}")
            print(f"  預計節省: {format_size(total_saved)}")
    else:
        print("\n🗑️  正在清理...")

        # 先嘗試標準清理
        result = run_uv_command(["cache", "clean"], check=False)
        if result and result.returncode == 0:
            print("✅ 標準 Cache 清理完成")
        else:
            print("⚠️  標準清理失敗，使用選擇性清理...")
            cleaned_count, skipped_count, total_saved = clean_cache_selective(
                cache_dir, dry_run=False
            )

            print("\n📊 清理結果:")
            print(f"  已清理檔案: {cleaned_count}")
            print(f"  跳過檔案: {skipped_count}")
            print(f"  節省空間: {format_size(total_saved)}")

            if skipped_count > 0:
                print(f"\n💡 提示: {skipped_count} 個檔案正在使用中，已跳過")
                print("   建議關閉相關程序後重新執行清理")

        # 顯示清理後的大小
        if cache_dir.exists():
            after_size = get_cache_size(cache_dir)
            saved_size = before_size - after_size
            print("\n📈 總體效果:")
            print(f"  清理前: {format_size(before_size)}")
            print(f"  清理後: {format_size(after_size)}")
            print(f"  實際節省: {format_size(saved_size)}")
        else:
            print(f"  節省空間: {format_size(before_size)}")


def force_clean_cache():
    """強制清理 cache（關閉相關程序後）"""
    print("🔥 強制清理模式")
    print("=" * 50)
    print("⚠️  警告：此模式會嘗試關閉可能使用 cache 的程序")

    confirm = input("確定要繼續嗎？(y/N): ")
    if confirm.lower() != "y":
        print("❌ 已取消")
        return

    cache_dir = get_cache_dir()
    if not cache_dir.exists():
        print("Cache 目錄不存在")
        return

    before_size = get_cache_size(cache_dir)
    print(f"清理前大小: {format_size(before_size)}")

    # 嘗試關閉可能的 uvx 程序
    print("\n🔍 檢查相關程序...")
    try:
        import psutil

        killed_processes = []
        for proc in psutil.process_iter(["pid", "name", "cmdline"]):
            try:
                if proc.info["name"] and any(
                    name in proc.info["name"].lower()
                    for name in ["uvx", "uv.exe", "python.exe"]
                ):
                    cmdline = " ".join(proc.info["cmdline"] or [])
                    if "mcp-feedback-enhanced" in cmdline or "uvx" in cmdline:
                        print(
                            f"  終止程序: {proc.info['name']} (PID: {proc.info['pid']})"
                        )
                        proc.terminate()
                        killed_processes.append(proc.info["pid"])
            except (psutil.NoSuchProcess, psutil.AccessDenied):
                pass

        if killed_processes:
            print(f"  已終止 {len(killed_processes)} 個程序")
            import time

            time.sleep(2)  # 等待程序完全關閉
        else:
            print("  未發現相關程序")

    except ImportError:
        print("  無法檢查程序（需要 psutil），繼續清理...")

    # 再次嘗試標準清理
    print("\n🗑️  執行清理...")
    result = run_uv_command(["cache", "clean"], check=False)
    if result and result.returncode == 0:
        print("✅ 強制清理成功")
    else:
        print("⚠️  標準清理仍然失敗，使用檔案級清理...")
        cleaned_count, skipped_count, total_saved = clean_cache_selective(
            cache_dir, dry_run=False
        )
        print(f"  清理檔案: {cleaned_count}, 跳過: {skipped_count}")

    # 顯示結果
    after_size = get_cache_size(cache_dir)
    saved_size = before_size - after_size
    print("\n📈 清理結果:")
    print(f"  節省空間: {format_size(saved_size)}")


def main():
    parser = argparse.ArgumentParser(description="UV Cache 清理工具")
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument("--size", action="store_true", help="顯示 cache 大小資訊")
    group.add_argument(
        "--dry-run", action="store_true", help="預覽清理內容（不實際清理）"
    )
    group.add_argument("--clean", action="store_true", help="執行 cache 清理")
    group.add_argument(
        "--force", action="store_true", help="強制清理（會嘗試關閉相關程序）"
    )

    args = parser.parse_args()

    if args.size:
        show_cache_info()
    elif args.dry_run:
        clean_cache(dry_run=True)
    elif args.clean:
        clean_cache(dry_run=False)
    elif args.force:
        force_clean_cache()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
回饋結果資料模型

定義回饋收集的資料結構，用於 Web UI 與後端的資料傳輸。
"""

from typing import TypedDict


class FeedbackResult(TypedDict):
    """回饋結果的型別定義"""

    command_logs: str
    interactive_feedback: str
    images: list[dict]



"""
MCP Feedback Enhanced 工具模組
============================

提供各種工具類和函數，包括錯誤處理、資源管理等。
"""

from .error_handler import ErrorHandler, ErrorType
from .resource_manager import (
    ResourceManager,
    cleanup_all_resources,
    create_temp_dir,
    create_temp_file,
    get_resource_manager,
    register_process,
)


__all__ = [
    "ErrorHandler",
    "ErrorType",
    "ResourceManager",
    "cleanup_all_resources",
    "create_temp_dir",
    "create_temp_file",
    "get_resource_manager",
    "register_process",
]
